import datetime
import json
import pytz

from django.db.utils import IntegrityError

from taggit.models import Tag

from . import FetchError
from ..models import Photo, Photoset, User


# These classes are passed JSON data from the Flickr API and create/update
# objects based on that.
#
# They don't do any fetching of data or files themselves.

# CLASSES HERE:
#
# SaveUtilsMixin
#
# UserSaver
# PhotoSaver
# PhotosetSaver


class SaveUtilsMixin(object):
    """Handy utility methods used by the *Saver classes."""

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _api_datetime_to_datetime(self, api_time, timezone_id=None):
        """Change a text datetime from the API to a datetime with timezone.
        api_time is a string like "1956-01-01 00:00:00".
        timezone_id is a TZ timezone name like 'Africa/Accra'
        """
        if timezone_id is None or timezone_id == '':
            timezone_id = 'Etc/UTC'
        tz = pytz.timezone(timezone_id)
        return datetime.datetime.strptime(
                            api_time, '%Y-%m-%d %H:%M:%S').replace(tzinfo=tz)

    def _unixtime_to_datetime(self, api_time):
        """Change a text unixtime from the API to a datetime with timezone.
        api_time is a string or int like "1093459273".
        """
        return datetime.datetime.utcfromtimestamp(
                                        int(api_time)).replace(tzinfo=pytz.utc)


class UserSaver(SaveUtilsMixin, object):
    """For creating/updating an individual User based on data from the API.

    Use like:

        UserSaver().save_user(data, fetch_time)
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save_user(self, user, fetch_time):
        """
        Keyword arguments:
        user -- A dict of the data about a user from the API's JSON.
        fetch_time -- A datetime.

        Returns the User object that was created or updated.
        """
        raw_json = json.dumps(user)

        defaults = {
            'fetch_time':   fetch_time,
            'raw':          raw_json,
            'nsid':         user['nsid'],
            'is_pro':       (int(user['ispro']) == 1),
            'iconserver':   user['iconserver'],
            'iconfarm':     user['iconfarm'],
            'username':     user['username']['_content'],
            'realname':     user['realname']['_content'],
            'location':     user['location']['_content'],
            'description':  user['description']['_content'],
            'photos_url':   user['photosurl']['_content'],
            'profile_url':  user['profileurl']['_content'],
            'photos_count': int(user['photos']['count']['_content']),
            'photos_first_date': self._unixtime_to_datetime(
                                user['photos']['firstdate']['_content']),
            'photos_first_date_taken': self._api_datetime_to_datetime(
                                user['photos']['firstdatetaken']['_content']),
            'timezone_id':  user['timezone']['timezone_id'],
        }

        if 'views' in user['photos']:
            # I think this might only be returned for the Account's own user.
            defaults['photos_views'] = \
                                    int(user['photos']['views']['_content'])

        user_obj, created = User.objects.update_or_create(
            nsid=user['nsid'], defaults=defaults
        )

        return user_obj


class PhotoSaver(SaveUtilsMixin, object):
    """For creating/updating an individual Photo based on data from the API.

    Use like:

        PhotoSaver().save_photo(data, fetch_time)
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save_photo(self, photo):
        """Takes a dict of photo data from the API and creates or updates a
        Photo object.

        Keyword arguments:
        photo -- The photo data, from several Flickr API calls.
                 Has keys:
                     'fetch_time': A datetime.
                     'user_obj': User object for this photo's owner.
                     'info': Results of a photos.getInfo call.
                     'exif': Results of a photos.getExif call.
                     'sizes': Results of a photos.getSizes call.

        Returns:
        The Photo object that was created or updated.
        """

        # photo['info']['urls'] = {
        #     'url': [ {'type':'photopage', '_content':'http...'} ]
        # }
        permalink = next(url for url in photo['info']['urls']['url'] if url['type'] == 'photopage')['_content']

        defaults = {
            # DittoItemModel fields
            'title':                photo['info']['title']['_content'],
            'permalink':            permalink,
            'is_private':           (photo['info']['visibility']['ispublic'] == 0),
            'fetch_time':           photo['fetch_time'],
            'post_time':            self._unixtime_to_datetime(
                                            photo['info']['dates']['posted']),
            'raw':                  json.dumps(photo['info']),

            # Photo fields
            'user':                 photo['user_obj'],
            'flickr_id':            photo['info']['id'],
            'description':          photo['info']['description']['_content'],
            'secret':               photo['info']['secret'],
            'original_secret':      photo['info']['originalsecret'],
            'server':               photo['info']['server'],
            'farm':                 photo['info']['farm'],
            'license':              photo['info']['license'],
            'rotation':             photo['info']['rotation'],
            'original_format':      photo['info']['originalformat'],
            'safety_level':         photo['info']['safety_level'],
            'has_people':           (int(photo['info']['people']['haspeople']) == 1),
            'last_update_time':     self._unixtime_to_datetime(
                                        photo['info']['dates']['lastupdate']),
            'taken_time':           self._api_datetime_to_datetime(
                                                photo['info']['dates']['taken'],
                                                photo['user_obj'].timezone_id),
            'taken_granularity':    \
                            int(photo['info']['dates']['takengranularity']),
            'taken_unknown':        \
                            (int(photo['info']['dates']['takenunknown']) == 1),
            'view_count':           photo['info']['views'],
            'comment_count':        photo['info']['comments']['_content'],
            'media':                photo['info']['media'],

            # Location fields

            # Sizes fields
            'sizes_raw':            json.dumps(photo['sizes']),

            # EXIF fields
            'exif_raw':             json.dumps(photo['exif']),
            'exif_camera':          photo['exif']['camera'],
        }

        # Photos with no location have these fields missing entirely:
        if 'geoperms' in photo['info']:
            defaults['geo_is_private']      = (int(photo['info']['geoperms']['ispublic']) == 0)

        if 'location' in photo['info']:
            loc = photo['info']['location']
            defaults['latitude']            = loc['latitude']
            defaults['longitude']           = loc['longitude']
            defaults['location_accuracy']   = loc['accuracy']
            defaults['location_context']    = loc['context']
            if 'place_id' in loc:
                defaults['location_place_id']   = loc['place_id']
            if 'woeid' in loc:
                defaults['location_woeid']      = loc['woeid']
            if 'locality' in loc:
                defaults['locality_name']       = loc['locality']['_content']
                defaults['locality_place_id']   = loc['locality']['place_id']
                defaults['locality_woeid']      = loc['locality']['woeid']
            if 'county' in loc:
                defaults['county_name']         = loc['county']['_content']
                defaults['county_place_id']     = loc['county']['place_id']
                defaults['county_woeid']        = loc['county']['woeid']
            if 'region' in loc:
                defaults['region_name']         = loc['region']['_content']
                defaults['region_place_id']     = loc['region']['place_id']
                defaults['region_woeid']        = loc['region']['woeid']
            if 'country' in loc:
                defaults['country_name']        = loc['country']['_content']
                defaults['country_place_id']    = loc['country']['place_id']
                defaults['country_woeid']       = loc['country']['woeid']

        # The size labels for all possible sizes an image might have, that we
        # also have width/height parameters for on Photo:
        sizes = [v['label'] for k,v in Photo.PHOTO_SIZES.items()] + \
                [v['label'] for k,v in Photo.VIDEO_SIZES.items()]
        # We don't store width/height for these, so ignore them:
        sizes.remove('Square')
        sizes.remove('Large square')

        for size in photo['sizes']['size']:
            if size['label'] in sizes:
                # eg, 'Small 320' becomes 'small_320':
                name = size['label'].lower().replace(" ", "_")
                defaults[name+'_width'] = int(size['width'])
                defaults[name+'_height'] = int(size['height'])

        try:
            for e in photo['exif']['exif']:
                if e['tag'] == 'LensModel':
                    defaults['exif_lens_model'] = e['raw']['_content']
                elif e['tag'] == 'FNumber':
                    defaults['exif_aperture'] = e['clean']['_content']
                elif e['tag'] == 'ExposureTime':
                    defaults['exif_exposure'] = e['clean']['_content']
                elif e['tag'] == 'Flash':
                    defaults['exif_flash'] = e['raw']['_content']
                elif e['tag'] == 'FocalLength':
                    defaults['exif_focal_length'] = e['clean']['_content']
                elif e['tag'] == 'ISO':
                    defaults['exif_iso'] = e['raw']['_content']
        except KeyError:
            pass

        photo_obj, created = Photo.objects.update_or_create(
                flickr_id=photo['info']['id'],
                defaults=defaults
            )

        self._save_tags(photo_obj, photo['info']['tags']['tag'])

        return photo_obj

    def _save_tags(self, photo_obj, tags_data):
        """
        Adds/deletes tags for a photo.

        Required Arguments
          photo_obj: The Photo object we're altering tags for.
          photo_user_obj: The User whose photo this is (because they're most
                            likely to have created the tags.)
          tags_data: A list of dicts about the tags, straight from the API.
        """

        # The existing tag-photo relationships.
        tagged_photos = Photo.tags.through.objects.filter(
                                                   content_object=photo_obj)

        local_flickr_ids = set([])
        remote_flickr_ids = set([])

        # Get the Flickr IDs of all the current tag-photo relationships.
        for tagged_photo in tagged_photos:
            local_flickr_ids.add(tagged_photo.flickr_id)

        for tag in tags_data:
            remote_flickr_ids.add(tag['id'])

            if tag['id'] not in local_flickr_ids:

                # This tag isn't currently on the photo, so add it.
                try:
                    tag_obj, tag_created = Tag.objects.get_or_create(
                        slug=tag['_content'], defaults={ 'name':tag['raw'] }
                    )
                except IntegrityError:
                    # It's possible for there to be a tag with a different
                    # slug but the same name, which would cause an
                    # IntegrityError.
                    # In which case, just fetch the existing Tag by name:
                    tag_obj = Tag.objects.get(name=tag['raw'])

                # Who created this tag?
                if tag['author'] == photo_obj.user.nsid:
                    # The same person whose photo these tags are on.
                    user = photo_obj.user
                else:
                    # In theory we'll already have fetched and saved data for
                    # all authors of these tags when fetching this photo's
                    # data.
                    try:
                        user = User.objects.get(nsid=tag['author'])
                    except User.DoesNotExist:
                        raise FetchError("Tried to add a Tag authored by a Flickr user with NSID %s who doesn't exist in the DB." % tag['author'])

                pt_obj = Photo.tags.through(
                            flickr_id = tag['id'],
                            author = user,
                            machine_tag = (tag['machine_tag'] == "1"),
                            content_object = photo_obj,
                            tag = tag_obj
                        )
                pt_obj.save()

        flickr_ids_to_delete = local_flickr_ids.difference(remote_flickr_ids)

        # Finally, delete any tag-photo relationships which were identified
        # above as no longer on the photo on Flickr.
        for tagged_photo in tagged_photos:
            if tagged_photo.flickr_id in flickr_ids_to_delete:
                tagged_photo.delete()


class PhotosetSaver(SaveUtilsMixin, object):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save_photoset(self, photoset):
        """Takes a dict of photoset data from the API and creates or updates a
        Photoset object.

        It accepts a list of photo data, but will only add each photo to the
        photoset if we already have that photo's data in the DB.

        Keyword arguments:
        photoset -- Has keys:
                    'fetch_time': A datetime.
                    'photoset': The data about the photoset from the API.
                    'photos': The data about the set's photos from the API.
                    'user_obj': User object for this photoset's owner.

        Returns:
        The Photoset object that was created or updated.
        """

        ps = photoset['photoset']

        defaults = {
            'fetch_time':           photoset['fetch_time'],
            'user':                 photoset['user_obj'],
            'flickr_id':            ps['id'],
            'title':                ps['title']['_content'],
            'description':          ps['description']['_content'],
            'photo_count':          ps['photos'],
            'video_count':          ps['videos'],
            'view_count':           ps['count_views'],
            'comment_count':        ps['count_comments'],
            'last_update_time':     self._unixtime_to_datetime(
                                                            ps['date_update']),
            'flickr_created_time':  self._unixtime_to_datetime(
                                                            ps['date_create']),
            'raw':                  json.dumps(ps),
            'photos_raw':           json.dumps(photoset['photos']),
        }

        try:
            defaults['primary_photo'] = \
                                    Photo.objects.get(flickr_id=ps['primary'])
        except Photo.DoesNotExist:
            pass

        photoset_obj, created = Photoset.objects.update_or_create(
                flickr_id=ps['id'],
                defaults=defaults
            )

        if photoset_obj:
            # Add all the photoset's photos that we have in the DB to the
            # photoset object.
            photos = []
            for photo in photoset['photos']:
                try:
                    photos.append( Photo.objects.get(flickr_id=photo['id']) )
                except Photo.DoesNotExist:
                    pass

            # Sets/updates the SortedManyToMany field of the photoset's photos:
            photoset_obj.photos = photos

        return photoset_obj




from django.db import models

from taggit.managers import _TaggableManager


class PublicToreadManager(models.Manager):
    """Returns public Bookmarks from any of the Accounts marked 'to_read'."""
    def get_queryset(self):
        return super().get_queryset().filter(is_private=False).filter(to_read=True)


class ToreadManager(models.Manager):
    """Returns public AND PRIVATE Bookmarks from any of the Accounts marked
    'to_read'.
    """
    def get_queryset(self):
        return super().get_queryset().filter(to_read=True)


class _BookmarkTaggableManager(_TaggableManager):
    """Providing some extra features related to private Bookmarks and tags."""

    def most_common(self):
        """Gets the most commonly-used tags but:
            * Doesn't count tags on private Bookmarks
            * Doesn't show tags that start with '.' (Pinboard's private tags)
        Overriding django-taggit's standard `most_common()` method.
        """
        extra_filters = {
            'bookmark__is_private': False,
        }

        return self.get_queryset(extra_filters).exclude(name__startswith='.').annotate(
            num_times=models.Count(self.through.tag_relname())
        ).order_by('-num_times')

    def all(self):
        """Overriding the default self.all() so we can exclude the private tags
        Pinboard uses, ie, tags that start with '.'.
        And order the tags alphabetically.

        Use like `Bookmark.tags.all()`.
        """
        return self.get_queryset().exclude(name__startswith='.').order_by('name')

    def names(self):
        """Override default so we order by name."""
        return self.get_queryset().exclude(name__startswith='.').order_by('name').values_list('name', flat=True)




from django.conf.urls import url

from . import views


app_name = 'twitter'

urlpatterns = [
    url(
        regex=r"^$",
        view=views.HomeView.as_view(),
        name='home'
    ),
    url(
        regex=r"^likes/$",
        view=views.FavoriteListView.as_view(),
        name='favorite_list'
    ),
    url(
        regex=r"^(?P<screen_name>\w+)/$",
        view=views.UserDetailView.as_view(),
        name='user_detail'
    ),
    url(
        regex=r"^(?P<screen_name>\w+)/likes/$",
        view=views.AccountFavoriteListView.as_view(),
        name='account_favorite_list'
    ),
    url(
        regex=r"^(?P<screen_name>\w+)/(?P<twitter_id>\d+)/$",
        view=views.TweetDetailView.as_view(),
        name='tweet_detail'
    ),
]




from datetime import timedelta

from django.db import models
from django.http import Http404
from django.shortcuts import redirect
from django.utils.translation import ugettext as _
from django.views.generic import DetailView, TemplateView
from django.views.generic.detail import SingleObjectMixin

from ..core.utils import datetime_now
from ..core.views import PaginatedListView
from .models import Account, Album, Artist, Scrobble, Track


class AccountsMixin(object):
    """
    View Mixin for adding an `account_list` to context, with all Accounts in.
    And the total counts of Scrobbles, Albums, Artists and Tracks.
    """
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account_list'] = Account.objects.all()
        context['counts']   = {
            'albums':       Album.objects.all().count(),
            'artists':      Artist.objects.all().count(),
            'scrobbles':    Scrobble.objects.all().count(),
            'tracks':       Track.objects.all().count(),
        }
        return context


class HomeView(AccountsMixin, TemplateView):
    "Uses template tags to display charts, recent Scrobbles, etc."
    template_name = 'lastfm/home.html'


class ScrobbleListView(AccountsMixin, PaginatedListView):
    "A multi-page list of Scrobbles, most recent first."
    template_name = 'lastfm/scrobble_list.html'
    model = Scrobble

    def get_queryset(self):
        "Pre-fetch Artists and Tracks to reduce number of queries."
        qs = super(ScrobbleListView, self).get_queryset()
        return qs.prefetch_related('artist', 'track')


class ChartPaginatedListView(PaginatedListView):
    """
    For the Album, Artist and Track ListViews that are charts ordered by
    scrobble_count.
    """
    ordering = '-scrobble_count'

    def get_queryset(self):
        return self.get_queryset_with_counts()

    def get_queryset_with_counts(self):
        """
        Includes scrobble_counts with returned objects, and adds ability to
        limit results to a time period based on the supplied `days`.

        This was just get_queryset(), but then combining this class with the
        SingleAccountMixin didn't work; child classes were using
        SingleObjectMixin's get_queryset().

        So now, each child class has to call this method from its own
        get_queryset().
        """
        queryset = self.model._default_manager

        # This will contain filters on the queryset:
        qs_kwargs = {}

        days = self.get_days()  # eg 7, 30 or 'all'

        if days != 'all':
            time_ago = datetime_now() - timedelta(days=days)
            qs_kwargs['min_post_time'] = time_ago

        if hasattr(self, 'object') and isinstance(self.object, Account):
            # We need to filter the results to only get Tracks (or whatever)
            # scrobbled by this Account.
            qs_kwargs['account'] = self.object

        queryset = queryset.with_scrobble_counts(**qs_kwargs)

        ordering = (self.get_ordering(),)
        queryset = queryset.order_by(*ordering)

        return queryset

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['current_days'] = str(self.get_days())
        context['valid_days'] = self.get_valid_days()
        return context

    def get_valid_days(self):
        return ['7', '30', '90', '180', '365', 'all',]

    def get_days(self):
        """
        Returns the current number of recent days we're viewing.

        Returns an integer (number of days) or 'all'.
        """
        days = self.request.GET.get('days')
        if days and days in self.get_valid_days():
            try:
                return int(days)
            except ValueError:
                return days
        else:
            return 'all'


class AlbumListView(AccountsMixin, ChartPaginatedListView):
    "A multi-page chart of most-scrobbled Tracks."
    template_name = 'lastfm/album_list.html'
    model = Album


class ArtistListView(AccountsMixin, ChartPaginatedListView):
    "A multi-page chart of most-scrobbled Tracks."
    template_name = 'lastfm/artist_list.html'
    model = Artist


class TrackListView(AccountsMixin, ChartPaginatedListView):
    "A multi-page chart of most-scrobbled Tracks."
    template_name = 'lastfm/track_list.html'
    model = Track


class AlbumDetailView(DetailView):
    "A single Album by a particular Artist."
    model = Album

    def get_object(self, queryset=None):
        """
        Returns the object the view is displaying.

        This is based on DetailView's method.
        """
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        artist_slug = self.kwargs.get('artist_slug')
        album_slug = self.kwargs.get('album_slug')

        if artist_slug is None or album_slug is None:
            raise AttributeError("AlbumDetailView must be called with "
                     "artist_slug and album_slug")

        artist = Artist.objects.get(slug=artist_slug)
        queryset = queryset.filter(artist=artist, slug=album_slug)

        try:
            # Get the single item from the filtered queryset
            obj = queryset.get()
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj


class ArtistDetailView(DetailView):
    "One Artist. Uses a template tag to display a chart of their Tracks."
    model = Artist
    slug_url_kwarg = 'artist_slug'


class ArtistAlbumsView(DetailView):
    "One Artist. Uses a template tag to display a chart of their Albums."
    model = Artist
    slug_url_kwarg = 'artist_slug'
    template_name = 'lastfm/artist_albums.html'


class TrackDetailView(DetailView):
    "One Track by a particular Artist."
    model = Track

    def get_object(self, queryset=None):
        """
        Returns the object the view is displaying.

        This is based on DetailView's method.
        """
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        artist_slug = self.kwargs.get('artist_slug')
        track_slug = self.kwargs.get('track_slug')

        if artist_slug is None or track_slug is None:
            raise AttributeError("TrackDetailView must be called with "
                     "artist_slug and track_slug")

        artist = Artist.objects.get(slug=artist_slug)
        queryset = queryset.filter(artist=artist, slug=track_slug)

        try:
            # Get the single item from the filtered queryset
            obj = queryset.get()
        except queryset.model.DoesNotExist:
            raise Http404(_("No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj


class SingleAccountMixin(SingleObjectMixin):
    """Used for views that need data about an Account based on username in
    the URL.
    """
    slug_field = 'username'
    slug_url_kwarg = 'username'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=Account.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        "Add counts of things for this Account only."
        context = super().get_context_data(**kwargs)

        qs = Scrobble.objects.filter(account=self.object)

        context['counts'] = {
            'albums':     qs.exclude(album_id__isnull=True)\
                                .values('album_id').distinct().count(),
            'artists':    qs.values('artist_id').distinct().count(),
            'scrobbles':  qs.count(),
            'tracks':     qs.values('track_id').distinct().count(),
        }
        return context


class UserDetailView(SingleAccountMixin, DetailView):
    "Overview of the user; top 10s of everything."
    template_name = 'lastfm/user_detail.html'
    model = Account


class UserAlbumListView(SingleAccountMixin, ChartPaginatedListView):
    "Chart of Albums scrobbled by one user."
    template_name = 'lastfm/user_album_list.html'
    model = Album

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['album_list'] = context['object_list']
        return context

    def get_queryset(self):
        return self.get_queryset_with_counts()


class UserArtistListView(SingleAccountMixin, ChartPaginatedListView):
    "Chart of Artists scrobbled by one user."
    template_name = 'lastfm/user_artist_list.html'
    model = Artist

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['artist_list'] = context['object_list']
        return context

    def get_queryset(self):
        return self.get_queryset_with_counts()


class UserScrobbleListView(SingleAccountMixin, PaginatedListView):
    "All scrobbles by one user."
    template_name = 'lastfm/user_scrobble_list.html'
    model = Scrobble

    def get_queryset(self):
        """
        All Scrobbles by this Account.
        And pre-fetch Artists and Tracks to reduce number of queries.
        """
        queryset = super().get_queryset()
        return queryset.filter(account=self.object)\
                        .prefetch_related('artist', 'track')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['scrobble_list'] = context['object_list']
        return context


class UserTrackListView(SingleAccountMixin, ChartPaginatedListView):
    "Chart of Tracks scrobbled by one user."
    template_name = 'lastfm/user_track_list.html'
    model = Track

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['track_list'] = context['object_list']
        return context

    def get_queryset(self):
        return self.get_queryset_with_counts()




# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0025_auto_20151109_1651'),
    ]

    operations = [
        migrations.AlterField(
            model_name='user',
            name='created_at',
            field=models.DateTimeField(help_text='UTC time when this account was created on Twitter', blank=True, null=True),
        ),
    ]



# coding: utf-8
from django.db import models
try:
    from django.urls import reverse
except ImportError:
    # For Django 1.8
    from django.urls import reverse
from django.templatetags.static import static

from imagekit.cachefiles import ImageCacheFile
from sortedm2m.fields import SortedManyToManyField
from taggit.managers import TaggableManager
from taggit.models import TaggedItemBase

from . import app_settings
from . import imagegenerators
from . import managers
from ..core.models import DiffModelMixin, DittoItemModel, TimeStampedModelMixin


class Account(TimeStampedModelMixin, models.Model):
    user = models.ForeignKey('User', blank=True, null=True,
                                                    on_delete=models.SET_NULL)

    api_key = models.CharField(blank=True, max_length=255,
                                                        verbose_name='API Key')
    api_secret = models.CharField(blank=True, max_length=255,
                                                    verbose_name='API Secret')

    is_active = models.BooleanField(default=True,
                            help_text="If false, new Photos won't be fetched.")

    def __str__(self):
        if self.user:
            return str(self.user)
        else:
            return '%d' % self.pk

    class Meta:
        ordering = ['user__realname']

    def get_absolute_url(self):
        if self.user:
            return reverse('flickr:user_detail',
                        kwargs={'nsid': self.user.nsid})
        else:
            return ''

    def has_credentials(self):
        "Does this at least have something in its API fields? True or False"
        if self.api_key and self.api_secret:
            return True
        else:
            return False


class TaggedPhoto(TaggedItemBase):
    """
    Describes the relationship between a django-taggit Tag and a Photo.
    Flickr has various fields which are unique to this relationship, rather
    than the Tag itself.

    Get the TaggedPhoto objects for one tag object:
        tagged_photo = TaggedPhoto.objects.filter(tag_id=tag)

    Get the TaggedPhoto objects for a particular photo object:
        tagged_photo = TaggedPhoto.objects.filter(content_object=photo)

    Access the Tag info for a TaggedPhoto, eg:
        tagged_photo.tag.slug
    """
    flickr_id = models.CharField(max_length=200, verbose_name="Flickr ID",
                                            help_text="The tag's ID on Flickr")
    author = models.ForeignKey('User', on_delete=models.CASCADE)
    machine_tag = models.BooleanField(default=False)
    content_object = models.ForeignKey('Photo', on_delete=models.CASCADE,
                                related_name="%(app_label)s_%(class)s_items")

    class Meta:
        verbose_name = 'Photo/Tag Relationship'


class ExtraPhotoManagers(models.Model):
    """Managers to use in the Photo model, in addition to the defaults defined
    in DittoItemModel.
    These need to be here, rather than in the Photo model, or they will
    override those in DittoItemModel.
    """
    photo_objects = managers.PhotosManager()
    public_photo_objects = managers.PublicPhotosManager()

    class Meta:
        abstract = True


class Photo(DittoItemModel, ExtraPhotoManagers):

    ditto_item_name = 'flickr_photo'

    # The keys in this dict are what we use internally, for method names and
    # for the sizes of PhotoDownloads.
    # The 'label's are used in Flickr's API to identify sizes.
    # The 'generator's are used to create an image of that size from original
    # files.
    # See https://www.flickr.com/services/api/misc.urls.html
    PHOTO_SIZES = {
        'square': {
            'label':    'Square',
            'suffix':   's',   # Used in this size's flickr.com URL.
            'generator': imagegenerators.Square,
        },
        'large_square': {
            'label':    'Large square',
            'suffix':   'q',
            'generator': imagegenerators.LargeSquare,
        },
        'thumbnail': {
            'label':    'Thumbnail',
            'suffix':   't',
            'generator': imagegenerators.Thumbnail,
        },
        'small':    {
            'label':    'Small',
            'suffix':   'm',
            'generator': imagegenerators.Small,
        },
        'small_320':    {
            'label':    'Small 320',
            'suffix':   'n',
            'generator': imagegenerators.Small320,
        },
        'medium':    {
            'label':    'Medium',
            'suffix':   '',
            'generator': imagegenerators.Medium,
        },
        'medium_640':    {
            'label':    'Medium 640',
            'suffix':   'z',
            'generator': imagegenerators.Medium640,
        },
        # Only exist after March 1st 2012.
        'medium_800':    {
            'label':    'Medium 800',
            'suffix':   'c',
            'generator': imagegenerators.Medium800,
        },
        # Before May 25th 2010, large only exist for very large original images.
        'large':    {
            'label':    'Large',
            'suffix':   'b',
            'generator': imagegenerators.Large,
        },
        # Only exist after March 1st 2012.
        'large_1600':    {
            'label':    'Large 1600',
            'suffix':   'h',
            'generator': imagegenerators.Large1600,
        },
        # Only exist after March 1st 2012.
        'large_2048':    {
            'label':    'Large 2048',
            'suffix':   'k',
            'generator': imagegenerators.Large2048,
        },
        'original':    {
            'label':    'Original',
            'suffix':   'o',
        },
    }

    VIDEO_SIZES = {
        'mobile_mp4': {
            'label': 'Mobile MP4',
            'url_size': 'mobile',   # Used in this size's flickr.com URL.
        },
        'site_mp4': {
            'label': 'Site MP4',
            'url_size': 'site',
        },
        'hd_mp4': {
            'label': 'HD MP4',
            'url_size': 'hd',
        },
        'video_original': {
            'label': 'Video Original',
            'url_size': 'orig',
        },
    }

    # From
    # https://www.flickr.com/services/api/flickr.photos.licenses.getInfo.html
    LICENSES = (
        ('0', 'All Rights Reserved'),
        ('1', 'Attribution-NonCommercial-ShareAlike License'),
        ('2', 'Attribution-NonCommercial License'),
        ('3', 'Attribution-NonCommercial-NoDerivs License'),
        ('4', 'Attribution License'),
        ('5', 'Attribution-ShareAlike License'),
        ('6', 'Attribution-NoDerivs License'),
        ('7', 'No known copyright restrictions'),
        ('8', 'United States Government Work'),
        ('9', 'Public Domain Dedication (CC0)'),
        ('10', 'Public Domain Mark'),
        # Adding these so we'll at least have options for any future licenses:
        ('11', 'Unused'),
        ('12', 'Unused'),
        ('13', 'Unused'),
        ('14', 'Unused'),
        ('15', 'Unused'),
        ('16', 'Unused'),
        ('17', 'Unused'),
        ('18', 'Unused'),
        ('19', 'Unused'),
        ('20', 'Unused'),
    )

    # Used in ditto/flickr/templatetags/flickr.py
    LICENSE_URLS = {
        '0': '',
        '1': 'https://creativecommons.org/licenses/by-nc-sa/2.0/',
        '2': 'https://creativecommons.org/licenses/by-nc/2.0/',
        '3': 'https://creativecommons.org/licenses/by-nc-nd/2.0/',
        '4': 'https://creativecommons.org/licenses/by/2.0/',
        '5': 'https://creativecommons.org/licenses/by-sa/2.0/',
        '6': 'https://creativecommons.org/licenses/by-nd/2.0/',
        '7': 'https://www.flickr.com/commons/usage/',
        '8': 'http://www.usa.gov/copyright.shtml',
        '9': 'https://creativecommons.org/publicdomain/zero/1.0/',
        '10': 'https://creativecommons.org/publicdomain/mark/1.0/',
    }

    # https://www.flickr.com/services/api/flickr.photos.setSafetyLevel.html
    SAFETY_LEVELS = (
        (0, 'None'), # Some have 0 as a safety_level.
        (1, 'Safe'),
        (2, 'Moderate'),
        (3, 'Restricted'),
    )

    # From https://www.flickr.com/services/api/misc.dates.html
    DATE_GRANULARITIES = (
        (0, 'Y-m-d H:i:s'),
        (1, 'Unused'),
        (2, 'Unused'),
        (3, 'Unused'),
        (4, 'Y-m'),
        (5, 'Unused'),
        (6, 'Y'),
        (7, 'Unused'),
        (8, 'Circa...'),
        (9, 'Unused'),
        (10, 'Unused'),
    )

    MEDIA_TYPES = (
        ('photo', 'Photo'),
        ('video', 'Video'),
    )

    # From https://www.flickr.com/services/api/flickr.photos.search.html
    LOCATION_CONTEXTS = (
        (0, 'not defined'),
        (1, 'indoors'),
        (2, 'outdoors'),
    )

    # Properties inherited from DittoItemModel:
    #
    # title         (CharField)
    # permalink     (URLField)
    # summary       (CharField)
    # is_private    (BooleanField)
    # fetch_time    (DateTimeField, UTC)
    # post_time     (DateTimeField, UTC)
    # latitude      (DecimalField)
    # longitude     (DecimalField)
    # raw           (TextField)

    user = models.ForeignKey('User', on_delete=models.CASCADE)

    flickr_id = models.BigIntegerField(unique=True, db_index=True,
                                    help_text="ID of this photo on Flickr.")
    description = models.TextField(blank=True, help_text="Can contain HTML")

    secret = models.CharField(max_length=20)
    original_secret = models.CharField(max_length=20)
    server = models.CharField(max_length=20)
    farm = models.PositiveSmallIntegerField()

    license = models.CharField(max_length=50, choices=LICENSES)
    rotation = models.PositiveSmallIntegerField(default=0,
        help_text="Current clockwise rotation, in degrees, by which the smaller image sizes differ from the original image.")
    original_format = models.CharField(max_length=10, help_text="eg, 'png'")
    safety_level = models.PositiveSmallIntegerField(
                            default=SAFETY_LEVELS[0][0], choices=SAFETY_LEVELS)

    has_people = models.BooleanField(default=False,
                            help_text="Are there Flickr users in this photo?")

    last_update_time = models.DateTimeField(null=True, blank=True,
        help_text="The last time the photo, or any of its metadata (tags, comments, etc.) was modified on Flickr. UTC.")
    taken_time = models.DateTimeField(null=True, blank=True,
                                help_text="In the Flickr user's timezone.")
    taken_granularity = models.PositiveSmallIntegerField(
                default=DATE_GRANULARITIES[0][0], choices=DATE_GRANULARITIES)
    taken_unknown = models.BooleanField(default=False)
    taken_year = models.PositiveSmallIntegerField(
                                    null=True, blank=True, db_index=True,
                                    help_text="Set automatically on save")

    view_count = models.PositiveIntegerField(default=0,
                help_text="How many times this had been viewed when fetched")
    comment_count = models.PositiveIntegerField(default=0,
                        help_text="How many comments this had when fetched")

    media = models.CharField(default=MEDIA_TYPES[0][0], choices=MEDIA_TYPES,
                                                                max_length=10)

    # SIZES ##################################################################

    sizes_raw = models.TextField(blank=True,
            help_text="The raw JSON from the API - flickr.photos.getSizes.")

    thumbnail_width = models.PositiveSmallIntegerField(null=True, blank=True)
    thumbnail_height = models.PositiveSmallIntegerField(null=True, blank=True)
    small_width = models.PositiveSmallIntegerField(null=True, blank=True)
    small_height = models.PositiveSmallIntegerField(null=True, blank=True)
    small_320_width = models.PositiveSmallIntegerField(null=True, blank=True)
    small_320_height = models.PositiveSmallIntegerField(null=True, blank=True)
    medium_width = models.PositiveSmallIntegerField(null=True, blank=True)
    medium_height = models.PositiveSmallIntegerField(null=True, blank=True)
    medium_640_width = models.PositiveSmallIntegerField(null=True, blank=True)
    medium_640_height = models.PositiveSmallIntegerField(null=True, blank=True)
    medium_800_width = models.PositiveSmallIntegerField(null=True, blank=True)
    medium_800_height = models.PositiveSmallIntegerField(null=True, blank=True)
    large_width = models.PositiveSmallIntegerField(null=True, blank=True)
    large_height = models.PositiveSmallIntegerField(null=True, blank=True)
    large_1600_width = models.PositiveSmallIntegerField(null=True, blank=True)
    large_1600_height = models.PositiveSmallIntegerField(null=True, blank=True)
    large_2048_width = models.PositiveSmallIntegerField(null=True, blank=True)
    large_2048_height = models.PositiveSmallIntegerField(null=True, blank=True)
    original_width = models.PositiveSmallIntegerField(null=True, blank=True)
    original_height = models.PositiveSmallIntegerField(null=True, blank=True)

    # Video sizes; when media='video'.
    mobile_mp4_width = models.PositiveSmallIntegerField("Mobile MP4 width",
                                                        null=True, blank=True)
    mobile_mp4_height = models.PositiveSmallIntegerField("Mobile MP4 height",
                                                        null=True, blank=True)
    site_mp4_width = models.PositiveSmallIntegerField("Site MP4 width",
                                                        null=True, blank=True)
    site_mp4_height = models.PositiveSmallIntegerField("Site MP4 height",
                                                        null=True, blank=True)
    hd_mp4_width = models.PositiveSmallIntegerField("HD MP4 width",
                                                        null=True, blank=True)
    hd_mp4_height = models.PositiveSmallIntegerField("HD MP4 height",
                                                        null=True, blank=True)
    video_original_width = models.PositiveSmallIntegerField(
                                                        null=True, blank=True)
    video_original_height = models.PositiveSmallIntegerField(
                                                        null=True, blank=True)


    # LOCATION ###############################################################

    geo_is_private = models.BooleanField(default=False, null=False, blank=False,
        help_text="If true, the Photo's location info should not be displayed.")

    # https://www.flickr.com/services/api/flickr.photos.search.html
    location_accuracy = models.PositiveSmallIntegerField(
        null=True, blank=True, default=1,
        help_text="World is 1; Country is ~3; Region is ~6; City is ~11; Street is ~16.")
    # https://www.flickr.com/services/api/flickr.photos.search.html
    location_context = models.PositiveSmallIntegerField(
                    default=LOCATION_CONTEXTS[0][0], choices=LOCATION_CONTEXTS)
    location_place_id = models.CharField(blank=True, max_length=30)
    location_woeid = models.CharField(blank=True, max_length=30)

    locality_name = models.CharField(blank=True, max_length=255)
    locality_place_id = models.CharField(blank=True, max_length=30)
    locality_woeid = models.CharField(blank=True, max_length=30)
    county_name = models.CharField(blank=True, max_length=255)
    county_place_id = models.CharField(blank=True, max_length=30)
    county_woeid = models.CharField(blank=True, max_length=30)
    region_name = models.CharField(blank=True, max_length=255)
    region_place_id = models.CharField(blank=True, max_length=30)
    region_woeid = models.CharField(blank=True, max_length=30)
    country_name = models.CharField(blank=True, max_length=255)
    country_place_id = models.CharField(blank=True, max_length=30)
    country_woeid = models.CharField(blank=True, max_length=30)

    # EXIF ###################################################################

    # EXIF data comes from a separate query, so store its JSON here.
    exif_raw = models.TextField(blank=True,
            help_text="The raw JSON from the API from flickr.photos.getExif.")
    exif_camera = models.CharField(blank=True, max_length=50)
    exif_lens_model = models.CharField(blank=True, max_length=50,
            help_text="eg, 'E PZ 16-50mm F3.5-5.6 OSS'.")
    exif_aperture = models.CharField(blank=True, max_length=30,
            help_text="eg, 'f/13.0'.")
    exif_exposure = models.CharField(blank=True, max_length=30,
            help_text="eg, '0.01 sec (1/100)'.")
    exif_flash = models.CharField(blank=True, max_length=30,
            help_text="eg, 'Off, Did not fire'.")
    exif_focal_length = models.CharField(blank=True, max_length=10,
            help_text="eg, '38 mm.'")
    exif_iso = models.IntegerField(blank=True, null=True, help_text="eg, '100'.")

    # TODO: NOTES
    # TODO: PEOPLE

    tags = TaggableManager(blank=True, manager=managers._PhotoTaggableManager,
                                                        through=TaggedPhoto)

    def upload_path(self, filename):
        "Make path under MEDIA_ROOT where original files will be saved."
        # If NSID is '35034346050@N01', get '35034346050'
        nsid = self.user.nsid[ :self.user.nsid.index('@') ]

        # If NSID is '35034346050@N01'
        # then, 'flickr/60/50/35034346050/avatars/avatar_name.jpg'
        return '/'.join([
            app_settings.DITTO_FLICKR_DIR_BASE,
            nsid[-4:-2],
            nsid[-2:],
            self.user.nsid.replace('@',''),
            'photos',
            str(self.taken_time.date().strftime(
                                app_settings.DITTO_FLICKR_DIR_PHOTOS_FORMAT)),
            filename
        ])

    original_file = models.ImageField(upload_to=upload_path,
                                            null=False, blank=True, default='')
    video_original_file = models.FileField(upload_to=upload_path,
                                        null=False, blank=True, default='',
                                        help_text="Only present for Videos.")

    class Meta:
        ordering = ('-post_time',)

    def save(self, *args, **kwargs):
        if self.taken_time:
            self.taken_year = self.taken_time.year
        else:
            self.taken_year = None
        super().save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('flickr:photo_detail',
                kwargs={'nsid': self.user.nsid, 'flickr_id': self.flickr_id})

    def get_next_public_by_post_time(self):
        "The next public Photo by this User, ordered by post_time."
        try:
            return Photo.public_photo_objects.filter(post_time__gte=self.post_time, user=self.user).exclude(pk=self.pk).order_by('post_time')[:1].get()
        except:
            pass

    def get_previous_public_by_post_time(self):
        "The previous public Photo by this User, ordered by post_time."
        try:
            return Photo.public_photo_objects.filter(post_time__lte=self.post_time, user=self.user).exclude(pk=self.pk).order_by('-post_time')[:1].get()
        except:
            pass

    # Shortcuts:
    def get_next(self):
        return self.get_next_public_by_post_time()

    def get_previous(self):
        return self.get_previous_public_by_post_time()

    @property
    def account(self):
        "The Account whose photo this is, if any. Otherwise, None."
        try:
            return self.user.account_set.all()[0]
        except IndexError:
            return None

    @property
    def safety_level_str(self):
        "Returns the text version of the safety_level, eg 'Restricted'."
        levels = dict((x,y) for x,y in self.SAFETY_LEVELS)

        try:
            return levels[self.safety_level]
        except KeyError:
            return None

    @property
    def location_str(self):
        "eg 'Abbey Dore, Herefordshire, England, United Kingdom'."
        strs = [
            self.locality_name,
            self.county_name,
            self.region_name,
            self.country_name,
        ]
        return ', '.join(filter(None, strs))

    @property
    def has_exif(self):
        "Do we have any EXIF info to display?"
        props = ['exif_camera', 'exif_lens_model', 'exif_aperture',
                'exif_exposure', 'exif_flash', 'exif_focal_length', 'exif_iso',]
        has_exif = False

        for prop in props:
            if getattr(self, prop):
                has_exif = True
                break

        return has_exif

    # Properties for always-the-same widths/heights, to maintain consistency:

    @property
    def square_width(self):
        return 75

    @property
    def square_height(self):
        return 75

    @property
    def large_square_width(self):
        return 150

    @property
    def large_square_height(self):
        return 150

    @property
    def remote_original_url(self):
        """
        URL of the original image file on Flickr.com.
        Usually we'd use self.original_url but if we have
        DITTO_FLICKR_USE_LOCAL_MEDIA as True, then we still need to be able
        to get the remote original file URL somehow. So use this.
        """
        return self._remote_image_url('original')

    @property
    def remote_video_original_url(self):
        """
        Same as remote_original_url but for video.
        EXCEPT currently we can ONLY get the remote video URLs anyway.
        """
        return self._remote_video_url('video_original')

    def _image_url(self, size):
        """
        Helper for the photo url property methods.
        See https://www.flickr.com/services/api/misc.urls.html
        size -- One of the keys from self.PHOTO_SIZES.
        """
        if app_settings.DITTO_FLICKR_USE_LOCAL_MEDIA:
            return self._local_image_url(size)
        else:
            return self._remote_image_url(size)

    def _local_image_url(self, size):
        """
        Generate the URL of an image of a particular size, hosted locally,
        based on the original file (which must already be downloaded).
        """
        if self.original_file:
            if size == 'original':
                return self.original_file.url
            else:
                generator = self.PHOTO_SIZES[size]['generator']
                try:
                    image_generator = generator(source=self.original_file)
                    result = ImageCacheFile(image_generator)
                    return result.url
                except:
                    # We have an original file but something's wrong with it.
                    # Might be 0 bytes or something.
                    return static('img/original_error.jpg')
        else:
            # We haven't downloaded an original file for this Photo.
            return static('img/original_missing.jpg')

    def _remote_image_url(self, size):
        """
        Generate the URL of an image of a particular size, on flickr.com.
        size -- One of the keys from self.PHOTO_SIZES.
        """
        if size == 'original':
            return 'https://farm%s.static.flickr.com/%s/%s_%s_%s.%s' % (
                self.farm, self.server, self.flickr_id, self.original_secret,
                self.PHOTO_SIZES['original']['suffix'], self.original_format)
        else:
            size_ext = ''
            # Medium size doesn't have a letter suffix.
            if self.PHOTO_SIZES[size]['suffix']:
                size_ext = '_%s' % self.PHOTO_SIZES[size]['suffix']
            return 'https://farm%s.static.flickr.com/%s/%s_%s%s.jpg' % (
                self.farm, self.server, self.flickr_id, self.secret, size_ext)

    def _video_url(self, size):
        return self._remote_video_url(size)

    #def _local_video_url(self, size):
        #"""One day, if we work out how to translate our downloaded original
        #video files into something usable on the web in different sizes, then
        #we can have _video_url() call this if
        #app_settings.DITTO_FLICKR_USE_LOCAL_MEDIA is False.
        #"""
        #pass

    def _remote_video_url(self, size):
        """
        Helper for the video URL property methods, on Flickr.com.
        Returns None for photos, or a URL for videos like:
        https://www.flickr.com/photos/philgyford/25743649964/play/site/a8bd5ddf59/
        size -- One of the keys from self.VIDEO_SIZES.
        """
        if self.media == 'photo':
            return None
        else:
            if size == 'video_original':
                secret = self.original_secret
            else:
                secret = self.secret
            url_size = self.VIDEO_SIZES[size]['url_size']
            return '%splay/%s/%s/' % (self.permalink, url_size, secret)

    def __getattr__(self, name):
        """
        Enables us to use properties like 'small_320_url' or 'site_mp4_url'.
        Will call the correct internal helper method to return the URL.
        """
        try:
            # Split 'small_240_url' into ['small_240', 'url']
            size, attr = name.rsplit('_', 1)
        except ValueError:
            self._raise_attr_error(type(self).__name__, name)

        if attr == 'url':
            # eg video_original_url()
            if size in self.VIDEO_SIZES:
                return self._video_url(size)
            elif size in self.PHOTO_SIZES:
                return self._image_url(size)
            else:
                self._raise_attr_error(type(self).__name__, name)
        else:
            self._raise_attr_error(type(self).__name__, name)

    def _raise_attr_error(self, obj, attr):
        msg = "'{0}' object has no attribute '{1}'"
        raise AttributeError(msg.format(obj, attr))

    def _summary_source(self):
        "Used to make the `summary` property."
        return self.description


class Photoset(TimeStampedModelMixin, DiffModelMixin, models.Model):
    user = models.ForeignKey('User', on_delete=models.CASCADE)
    flickr_id = models.BigIntegerField(null=False, blank=False, unique=True,
                                                                db_index=True)
    primary_photo = models.ForeignKey('Photo', on_delete=models.SET_NULL,
                                null=True, related_name='primary_photosets')
    title = models.CharField(null=False, blank=False, max_length=255)
    description = models.TextField(null=False, blank=True,
                                                help_text="Can contain HTML")
    photo_count = models.PositiveIntegerField(null=False, blank=False,
            default=0, help_text="The number of photos in the set on Flickr")
    video_count = models.PositiveIntegerField(null=False, blank=False,
            default=0, help_text="The number of videos in the set on Flickr")
    view_count = models.PositiveIntegerField(default=0,
                help_text="How many times this had been viewed when fetched")
    comment_count = models.PositiveIntegerField(default=0,
                        help_text="How many comments this had when fetched")

    last_update_time = models.DateTimeField(null=True, blank=True,
                help_text="The last time the set was modified on Flickr. UTC.")
    flickr_created_time = models.DateTimeField(null=True, blank=True,
                        help_text="When the set was created on Flickr. UTC.")

    fetch_time = models.DateTimeField(null=True, blank=True,
                        help_text="The time the item's data was last fetched.")
    raw = models.TextField(blank=True,
                                    help_text="The raw JSON from the API.")
    photos_raw = models.TextField(blank=True,
                    help_text="The raw JSON from the API listing the photos.")

    # Returns ALL photos, public AND private.
    photos = SortedManyToManyField('Photo', related_name='photosets')

    def public_photos(self):
        "Returns only public photos."
        return self.photos.filter(is_private=False)

    def __str__(self):
        return self.title

    class Meta:
        ordering = ['-flickr_created_time']

    def get_absolute_url(self):
        return reverse('flickr:photoset_detail',
                kwargs={'nsid': self.user.nsid, 'flickr_id': self.flickr_id})

    @property
    def permalink(self):
        return 'https://www.flickr.com/photos/%s/albums/%s' % (
                                            self.user.nsid, self.flickr_id)


class User(TimeStampedModelMixin, DiffModelMixin, models.Model):
    nsid = models.CharField(null=False, blank=False, unique=True, db_index=True,
                                        max_length=50, verbose_name='NSID')
    is_pro = models.BooleanField(null=False, blank=False, default=False,
                                                    verbose_name='Is Pro?')
    iconserver = models.PositiveIntegerField(null=False, blank=False,
                                                                   default=0)
    iconfarm = models.PositiveIntegerField(null=False, blank=False)

    username = models.CharField(null=False, blank=False, unique=True,
                                                                  max_length=50)
    realname = models.CharField(null=False, blank=False, max_length=255)
    location = models.CharField(null=False, blank=True, max_length=255)
    description = models.TextField(null=False, blank=True,
                                                help_text="May contain HTML")

    photos_url = models.URLField(null=False, blank=False, max_length=255,
                                            verbose_name='Photos URL at Flickr')
    profile_url = models.URLField(null=False, blank=False, max_length=255,
                                        verbose_name='Avatar URL on Flickr')

    photos_count = models.PositiveIntegerField(null=False, blank=False,
                                                                      default=0)
    photos_views = models.PositiveIntegerField(null=False, blank=False,
                                                                      default=0)
    photos_first_date = models.DateTimeField(null=True, blank=False)
    photos_first_date_taken = models.DateTimeField(null=True, blank=False)

    # As on DittoItemModel:
    fetch_time = models.DateTimeField(null=True, blank=True,
                            help_text="The time the data was last fetched.")
    raw = models.TextField(null=False, blank=True,
                                    help_text="The raw JSON from the API.")
    timezone_id = models.CharField(null=False, blank=False, max_length=50,
                                            help_text="eg, 'Europe/London'.")

    def avatar_upload_path(self, filename):
        "Make path under MEDIA_ROOT where avatar file will be saved."
        # If NSID is '35034346050@N01', get '35034346050'
        nsid = self.nsid[ :self.nsid.index('@') ]

        # If NSID is '35034346050@N01'
        # then, 'flickr/60/50/35034346050/avatars/avatar_name.jpg'
        return '/'.join([
            app_settings.DITTO_FLICKR_DIR_BASE,
            nsid[-4:-2],
            nsid[-2:],
            self.nsid.replace('@',''),
            'avatars',
            filename
        ])

    avatar = models.ImageField(upload_to=avatar_upload_path,
                                            null=False, blank=True, default='')

    objects = models.Manager()
    # All Users that have Accounts:
    objects_with_accounts = managers.WithAccountsManager()

    def __str__(self):
        return self.realname

    class Meta:
        ordering = ['realname']

    def get_absolute_url(self):
        return reverse('flickr:user_detail', kwargs={'nsid': self.nsid})

    @property
    def name(self):
        return self.realname

    @property
    def permalink(self):
        return self.photos_url

    @property
    def avatar_url(self):
        try:
            return self.avatar.url
        except ValueError:
            return static('img/default_avatar.png')

    @property
    def original_icon_url(self):
        """URL of the avatar/profile pic at Flickr."""
        if self.iconserver:
            return 'https://farm%s.staticflickr.com/%s/buddyicons/%s.jpg' % \
                                    (self.iconfarm, self.iconserver, self.nsid)
        else:
            return 'https://www.flickr.com/images/buddyicon.gif'




from django.conf import settings


# Creating all the defaults for settings.
# In our code, if we want to use a DITTO_FLICKR_* setting we should import
# from here, not django.conf.settings.


DITTO_FLICKR_DIR_BASE = getattr(settings, 'DITTO_FLICKR_DIR_BASE', 'flickr')

DITTO_FLICKR_DIR_PHOTOS_FORMAT = getattr(settings,
                                'DITTO_FLICKR_DIR_PHOTOS_FORMAT', '%Y/%m/%d')

DITTO_FLICKR_USE_LOCAL_MEDIA = getattr(settings,
                                        'DITTO_FLICKR_USE_LOCAL_MEDIA', False)




# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-11 11:07
from __future__ import unicode_literals

import ditto.core.models
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('username', models.CharField(help_text="eg, 'rj'", max_length=30, unique=True)),
                ('realname', models.CharField(help_text="eg, 'Richard Jones'", max_length=30)),
                ('api_key', models.CharField(blank=True, max_length=255, verbose_name='API Key')),
                ('is_active', models.BooleanField(default=True, help_text="If false, new scrobbles won't be fetched.")),
            ],
            options={
                'ordering': ['username'],
            },
        ),
        migrations.CreateModel(
            name='Album',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('name', models.TextField()),
                ('slug', models.TextField(db_index=True)),
                ('mbid', models.CharField(blank=True, help_text='MusicBrainz Identifier', max_length=36, verbose_name='MBID')),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Artist',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('name', models.CharField(max_length=255)),
                ('slug', models.TextField(unique=True)),
                ('mbid', models.CharField(blank=True, help_text='MusicBrainz Identifier', max_length=36, verbose_name='MBID')),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Scrobble',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('title', models.CharField(blank=True, max_length=255)),
                ('permalink', models.URLField(blank=True, help_text="URL of the item on the service's website.")),
                ('summary', models.CharField(blank=True, help_text="eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc. No HTML.", max_length=255)),
                ('is_private', models.BooleanField(default=False, help_text='If true, this item will not be shown on public-facing pages.')),
                ('fetch_time', models.DateTimeField(blank=True, help_text="The time the item's data was last fetched.", null=True)),
                ('post_time', models.DateTimeField(blank=True, help_text='The time the item was originally posted/created on its service.', null=True)),
                ('latitude', models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True)),
                ('longitude', models.DecimalField(blank=True, decimal_places=6, max_digits=9, null=True)),
                ('raw', models.TextField(blank=True, help_text='eg, the raw JSON from the API.')),
                ('account', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='lastfm.Account')),
                ('album', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='scrobbles', to='lastfm.Album')),
                ('artist', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='scrobbles', to='lastfm.Artist')),
            ],
            options={
                'ordering': ['-post_time'],
            },
            bases=(ditto.core.models.DiffModelMixin, models.Model),
        ),
        migrations.CreateModel(
            name='Track',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('time_created', models.DateTimeField(auto_now_add=True, help_text='The time this item was created in the database.')),
                ('time_modified', models.DateTimeField(auto_now=True, help_text='The time this item was last saved to the database.')),
                ('name', models.TextField()),
                ('slug', models.TextField(db_index=True)),
                ('mbid', models.CharField(blank=True, help_text='MusicBrainz Identifier', max_length=36, verbose_name='MBID')),
                ('artist', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='tracks', to='lastfm.Artist')),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.AddField(
            model_name='scrobble',
            name='track',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='scrobbles', to='lastfm.Track'),
        ),
        migrations.AddField(
            model_name='album',
            name='artist',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='albums', to='lastfm.Artist'),
        ),
    ]



# coding: utf-8
from .import FetchTwitterCommand
from ...fetch.fetchers import RecentTweetsFetcher


class Command(FetchTwitterCommand):
    """Fetches tweets from Twitter.

    Fetch recent tweets since the last fetch, from all accounts:
    ./manage.py fetch_twitter_tweets --recent=new

    Fetch 100 most recent tweets, from all accounts:
    ./manage.py fetch_twitter_tweets --recent=100

    Fetch recent tweets since the last fetch, from one account:
    ./manage.py fetch_twitter_tweets --recent=new --account=philgyford
    """

    help = "Fetches recent Tweets from Twitter"

    recent_help = 'Fetches the most recent Tweets, eg "100" or "new".'

    def fetch_tweets(self, screen_name, count):
        return RecentTweetsFetcher(screen_name=screen_name).fetch(count=count)





# coding: utf-8
import re

from django.utils.html import urlize

from ttp import ttp
from twython import Twython


def htmlify_description(json_data):
    """Passed the raw JSON data about a User from Twitter's API, it returns an
    HTMLified version of the User's description.
    * Replaces t.co URLs with clickable, full links.
    * Makes #hashtags into clickable links.
    * Makes @usernames into clickable links.

    Different to htmlify_tweet() because:

        * Twitter user data only includes entities for urls, not hashtags etc.
          https://twittercommunity.com/t/why-do-user-entities-have-only-urls-field-and-not-others/59181

        * So we manually make the t.co links into their full, clickable version.
        * And then use twitter-text-python to linkify everything else.
    """

    # I don't think users in the Twitter archive JSON have description
    # elements:
    try:
        desc = json_data['description']
    except KeyError:
        return ''

    # Make t.co URLs into their original URLs, clickable.
    if 'entities' in json_data and 'description' in json_data['entities']:
        entities = json_data['entities']['description']

        if 'urls' in entities:
            for entity in entities['urls']:
                start, end = entity['indices'][0], entity['indices'][1]
                shown_url = entity['display_url']
                link_url = entity['expanded_url']

                url_html = '<a href="%s" rel="external">%s</a>'
                desc = desc.replace(json_data['description'][start:end],
                                            url_html % (link_url, shown_url))

    # Make #hashtags and @usernames clickable.
    parser = ttp.Parser()
    parsed = parser.parse(desc)

    return parsed.html


def htmlify_tweet(json_data):
    """Passed the raw JSON data about a Tweet from Twitter's API, it returns
    an HTMLified version of the Tweet's text. It:
    * Replaces linebreaks with '<br>'s.
    * Replaces @mentions with clickable @mentions.
    * Replaces #hashtags with clickable #hashtags.
    * Replaces $symbols with clickable $symbols.
    * Replaces t.co URLs with clickable, full links.
    """

    # Temporary, until Twython.html_for_tweet() can handle tweets with
    # 'full_text' attributes.
    if 'full_text' in json_data:
        json_data['text'] = json_data['full_text']

    # Some Tweets (eg from a downloaded archive) don't have entities['symbols']
    # which Twython.html_for_tweet() currently expects.
    # Not needed once github.com/ryanmcgrath/twython/pull/451 is in Twython.
    if 'entities' in json_data and 'symbols' not in json_data['entities']:
        json_data['entities']['symbols'] = []

    # This does most of the work for us:
    # https://twython.readthedocs.org/en/latest/usage/special_functions.html#html-for-tweet
    html = Twython.html_for_tweet(
                    json_data, use_display_url=True, use_expanded_url=False)

    # Need to do some tidying up:

    try:
        ents = json_data['entities']
    except KeyError:
        ents = {}

    urls_count = len(ents['urls']) if 'urls' in ents else 0
    media_count = len(ents['media']) if 'media' in ents else 0
    hashtags_count = len(ents['hashtags']) if 'hashtags' in ents else 0
    symbols_count = len(ents['symbols']) if 'symbols' in ents else 0
    user_mentions_count = len(ents['user_mentions']) if 'user_mentions' in ents else 0

    # Replace the classes Twython adds with rel="external".
    html = html.replace('class="twython-hashtag"', 'rel="external"')
    html = html.replace('class="twython-mention"', 'rel="external"')
    html = html.replace('class="twython-media"', 'rel="external"')
    html = html.replace('class="twython-symbol"', 'rel="external"')

    # Twython uses the t.co URLs in the anchor tags.
    # We want to replace those with the full original URLs.
    # And replace the class it adds with rel="external".
    if (urls_count + media_count) > 0:
        if urls_count > 0:
            for url in ents['urls']:
                html = html.replace(
                        '<a href="%s" class="twython-url">' % url['url'],
                        '<a href="%s" rel="external">' % url['expanded_url']
                    )

    if media_count > 0:
        # Remove any media links, as we'll make the photos/movies visible in
        # the page. All being well.
        for item in ents['media']:
            html = html.replace('<a href="%s" rel="external">%s</a>' % \
                                        (item['url'], item['display_url']),
                                '')

    if (urls_count + media_count + hashtags_count + symbols_count + user_mentions_count) == 0:
        # Older Tweets might contain links but have no 'urls'/'media' entities.
        # So just make their links into clickable links:
        # But don't do this for newer Tweets which have an entities element,
        # or we'll end up trying to make links from, say user_mentions we
        # linked earlier.
        html = urlize(html)

    # Replace newlines with <br>s
    html = re.sub(r'\n', '<br>', html.strip())

    return html




# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-03 13:02
from __future__ import unicode_literals

from django.db import migrations


# This moves all the data from Media.tweet into Media.tweets.
# Because we're moving from a ForeignKey field (tweet) to a
# ManyToManyField (tweets).


def make_many_tweets(apps, schema_editor):
    """
        Adds the Tweet object in Media.tweet to the
        many-to-many relationship in Media.tweets
    """
    Media = apps.get_model('twitter', 'Media')

    for media in Media.objects.all():
        media.tweets.add(media.tweet)


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0039_auto_20160603_1301'),
    ]

    operations = [
        migrations.RunPython(make_many_tweets),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0005_auto_20160414_1427'),
    ]

    operations = [
        migrations.RenameField(
            model_name='photo',
            old_name='large_2400_height',
            new_name='large_2048_height',
        ),
        migrations.RenameField(
            model_name='photo',
            old_name='large_2400_width',
            new_name='large_2048_width',
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0004_auto_20160414_1120'),
    ]

    operations = [
        migrations.RenameField(
            model_name='photo',
            old_name='height_mp4_hd',
            new_name='hd_mp4_height',
        ),
        migrations.RenameField(
            model_name='photo',
            old_name='width_mp4_hd',
            new_name='hd_mp4_width',
        ),
        migrations.RenameField(
            model_name='photo',
            old_name='height_mp4_mobile',
            new_name='mobile_mp4_height',
        ),
        migrations.RenameField(
            model_name='photo',
            old_name='width_mp4_mobile',
            new_name='mobile_mp4_width',
        ),
        migrations.RenameField(
            model_name='photo',
            old_name='height_mp4_site',
            new_name='site_mp4_height',
        ),
        migrations.RenameField(
            model_name='photo',
            old_name='width_mp4_site',
            new_name='site_mp4_width',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_b',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_c',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_h',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_k',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_m',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_n',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_o',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_t',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_video_original',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='height_z',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_b',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_c',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_h',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_k',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_m',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_n',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_o',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_t',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_video_original',
        ),
        migrations.RemoveField(
            model_name='photo',
            name='width_z',
        ),
        migrations.AddField(
            model_name='photo',
            name='large_1600_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='large_1600_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='large_2400_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='large_2400_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='large_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='large_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='medium_640_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='medium_640_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='medium_800_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='medium_800_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='medium_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='medium_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='original_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='original_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='small_320_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='small_320_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='small_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='small_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='thumbnail_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='thumbnail_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='video_original_height',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
        migrations.AddField(
            model_name='photo',
            name='video_original_width',
            field=models.PositiveSmallIntegerField(null=True, blank=True),
        ),
    ]



import argparse

from django.core.management.base import BaseCommand, CommandError

from ....core.management.commands import DittoBaseCommand
from ...models import Account, User


class FetchCommand(DittoBaseCommand):
    """
    Parent for all classes that fetch some things from Flickr. Photos,
    Photosets, Files, etc.
    """

    def add_arguments(self, parser):
        "All children will have the --account option."
        super().add_arguments(parser)

        parser.add_argument(
            '--account',
            action='store',
            default=False,
            help='The NSID of the Flickr User associated with the one Account to fetch for.',
        )


class FetchPhotosCommand(FetchCommand):

    # What we're fetching:
    singular_noun = 'Photo'
    plural_noun = 'Photos'

    # Child classes should supply some help text for the --days argument:
    days_help = ""

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            '--days',
            action='store',
            default=False,
            help=self.days_help
        )

    def handle(self, *args, **options):

        # We might be fetching for a specific account or all (None).
        nsid = options['account'] if options['account'] else None;

        if options['days']:
            # Will be either 'all' or a number; make the number an int.
            if options['days'].isdigit():
                options['days'] = int(options['days'])
            elif options['days'] != 'all':
                raise CommandError("--days should be an integer or 'all'.")

            results = self.fetch_photos(nsid, options['days'])
            self.output_results(results, options.get('verbosity', 1))
        elif options['account']:
            raise CommandError("Specify --days as well as --account.")
        else:
            raise CommandError("Specify --days , eg --days=3 or --days=all.")

    def fetch_photos(self, nsid, days):
        """Child classes should override this method to call a method that
        fetches photos and returns results, eg:
            return RecentPhotosMultiAccountFetcher(nsid=nsid).fetch(days=days)
        """
        return {}




# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-03 12:24
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0037_user_avatar'),
    ]

    operations = [
        migrations.AddField(
            model_name='tweet',
            name='retweeted_status_id',
            field=models.BigIntegerField(blank=True, help_text='The ID of the retweeted Tweet, if any', null=True),
        ),
    ]



from . import FetchError
from .fetch import Fetch, FetchFiles, FetchTweetsRecent, FetchTweetsFavorite,\
        FetchTweets, FetchUsers, FetchVerify
from ..models import Account


# The classes to call to fetch data from the API to create/update objects.
# Each one can fetch a kind of thing for ONE OR MORE Accounts.
#
# They call the classes in fetch.fetch.* to do the actual API calling.
#
# In general (FilesFetcher excepted), use them something like this:
#
#   fetcher = RecentTweetsFetcher(screen_name='philgyford')
#   results = fetcher.fetch(count=20)

# CLASSES HERE:
#
# TwitterFetcher
#   VerifyFetcher
#   UsersFetcher
#   TweetsFetcher
#   RecentTweetsFetcher
#   FavoriteTweetsFetcher
# FilesFetcher


class TwitterFetcher(object):
    """Parent class for children that will call the Twitter API to fetch data
    for one or several Accounts.

    Use like:
        fetcher = ChildTwitterFetcher(screen_name='philgyford')
        results = fetcher.fetch()

    Or, for all accounts:
        fetcher = ChildTwitterFetcher()
        results = fetcher.fetch()

    Child classes should at least override:
        _get_account_fetcher()
    """

    def __init__(self, screen_name=None):
        """Keyword arguments:
        screen_name -- of the one Account to get, or None for all Accounts.

        Raises:
        FetchError if passed a screen_name there is no Account for.
        """
        # Sets self.accounts:
        self._set_accounts(screen_name)

        # Will be a list of dicts that we return detailing succes/failure
        # results, one dict per account we've fetched for. eg:
        # [ {'account': 'thescreename', 'success': True, 'fetched': 200} ]
        self.return_values = []

    def fetch(self, **kwargs):
        """Fetch data for one or more Accounts.

        Returns:
        A list of dicts, one dict per Account, containing data about
        success/failure.
        """
        for account in self.accounts:
            accountFetcher = self._get_account_fetcher(account)
            return_value = accountFetcher.fetch(**kwargs)
            self._add_to_return_values(return_value)

        return self.return_values

    def _get_account_fetcher(self, account):
        """Should be changed for each child class.
        Should return an instance of a child of Fetch().

        Keyword arguments:
        account -- An Account object.
        """
        return Fetch(account)

    def _add_to_return_values(self, return_value):
        """Add return_value to the list in self.return_values."""
        self.return_values.append(return_value)

    def _set_accounts(self, screen_name=None):
        """Sets self.accounts to all Accounts or just one.

        Keyword arguments:
        screen_name -- of the one Account to get, or None for all Accounts.

        Raises:
        FetchError if passed a screen_name there is no Account for, or if none
            of the requested account(s) are marked as is_active.
        """
        if screen_name is None:
            accounts = Account.objects.filter(is_active=True)
            if (len(accounts) == 0):
                raise FetchError("No active Accounts were found to fetch.")
        else:
            try:
                accounts = [Account.objects.get(user__screen_name=screen_name)]
            except Account.DoesNotExist:
                raise FetchError("There is no Account in the database with a screen_name of '%s'" % screen_name)
            else:
                if accounts[0].is_active == False:
                    raise FetchError("The '%s' Account is marked as inactive." % screen_name)

        self.accounts = accounts


class VerifyFetcher(TwitterFetcher):
    """Calls verify_credentials for one/all Accounts.

    If an Account verifies OK, its Twitter User data is fetched and its User
    is created/updated in the databse.

    Usage (or omit screen_name for all Accounts):
        fetcher = VerifyFetcher(screen_name='aScreenName')
        results = fetcher.fetch()
    """

    def fetch(self):
        "No special arguments."
        return super().fetch()

    def _get_account_fetcher(self, account):
        return FetchVerify(account)


class UsersFetcher(TwitterFetcher):
    """Fetches data for a list of Twitter users, based on their ID.

    A screen_name for an Account is required, as we need to fetch the users
    using the API credentials from an Account.

    Usage:
        fetcher = UsersFetcher(screen_name='aScreenName')
        results = fetcher.fetch(ids=[123456,9876,])
    """

    def fetch(self, ids=[]):
        """
        Keyword arguments:
        ids -- A list of Twitter user IDs to fetch and store data for.
        """
        return super().fetch(ids=ids)

    def _get_account_fetcher(self, account):
        return FetchUsers(account)


class TweetsFetcher(TwitterFetcher):
    """Fetches data for a list of Tweets, based on their ID.

    A screen_name for an Account is required, as we need to fetch the Tweets
    using the API credentials from an Account.

    Usage:
        fetcher = TweetFetcher(screen_name='aScreenName')
        results = fetcher.fetch(ids=[123456,9876,])
    """

    def fetch(self, ids=[]):
        """
        Keyword arguments:
        ids -- A list of Twitter Tweet IDs to fetch and store data for.
        """
        return super().fetch(ids=ids)

    def _get_account_fetcher(self, account):
        return FetchTweets(account)


class RecentTweetsFetcher(TwitterFetcher):
    """Fetches the most recent tweets for one/all Accounts.

    Will fetch tweets since the last fetch.

    Usage (or omit screen_name for all Accounts):
        fetcher = RecentTweetsFetcher(screen_name='aScreenName')
        results = fetcher.fetch(count=200) # or count='new'
    """

    def fetch(self, count='new'):
        """
        Keyword arguments:
        count -- Either 'new' (to fetch all tweets since the last time), or a
                number (eg, 100), to fetch that many of the most recent tweets,
                per Account.
        """
        return super().fetch(count=count)

    def _get_account_fetcher(self, account):
        return FetchTweetsRecent(account)


class FavoriteTweetsFetcher(TwitterFetcher):
    """Fetches tweets favorited by one/all Accounts, and associate each one
    with the Accounts' twitter User.

    Will fetch favorites since the last fetch.

    Usage (or omit screen_name for all Accounts):
        fetcher = FavoriteTweetsFetcher(screen_name='aScreenName')
        results = fetcher.fetch(count=200) # or count='new'
    """

    def fetch(self, count='new'):
        """
        Keyword arguments:
        count -- Either 'new' (to fetch all tweets since the last time), or a
                number (eg, 100), to fetch that many of the most recent
                favorite tweets, per Account.
        """
        return super().fetch(count=count)

    def _get_account_fetcher(self, account):
        return FetchTweetsFavorite(account)


class FilesFetcher(object):
    """
    Use like:
        fetcher = FilesFetcher()
        results = fetcher.fetch()
    or:
        results = fetcher.fetch(fetch_all=True)

    Doesn't do much - simply to preserve a similar interface to the other
    *Fetcher() classes that use the API and Accounts.
    """

    def __init__(self):
        self.return_values = []

    def fetch(self, fetch_all=False):
        results = FetchFiles().fetch(fetch_all=fetch_all)

        # Return a list to behave similar to the other *Fetcher() classes that
        # can deal with multiple Accounts.
        return [ results ]





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0032_auto_20160414_1637'),
    ]

    operations = [
        migrations.AlterField(
            model_name='media',
            name='aspect_ratio',
            field=models.CharField(help_text='eg, "4:3" or "16:9"', max_length=5, blank=True),
        ),
        migrations.AlterField(
            model_name='media',
            name='dash_url',
            field=models.URLField(verbose_name='MPEG-DASH URL', default='', blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='media',
            name='mp4_bitrate_1',
            field=models.PositiveIntegerField(help_text='Lowest bitrate', null=True, verbose_name='MP4 Bitrate (1)', blank=True),
        ),
        migrations.AlterField(
            model_name='media',
            name='mp4_bitrate_2',
            field=models.PositiveIntegerField(help_text='Medium bitrate', null=True, verbose_name='MP4 Bitrate (2)', blank=True),
        ),
        migrations.AlterField(
            model_name='media',
            name='mp4_bitrate_3',
            field=models.PositiveIntegerField(help_text='Highest bitrate', null=True, verbose_name='MP4 Bitrate (3)', blank=True),
        ),
        migrations.AlterField(
            model_name='media',
            name='mp4_url_1',
            field=models.URLField(help_text='Lowest bitrate', verbose_name='MP4 URL (1)', default='', blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='media',
            name='mp4_url_2',
            field=models.URLField(help_text='Medium bitrate', verbose_name='MP4 URL (2)', default='', blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='media',
            name='mp4_url_3',
            field=models.URLField(help_text='Highest bitrate', verbose_name='MP4 URL (3)', default='', blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='media',
            name='webm_url',
            field=models.URLField(verbose_name='WebM URL', default='', blank=True),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='media',
            name='xmpeg_url',
            field=models.URLField(verbose_name='X-MPEG URL', default='', blank=True),
            preserve_default=False,
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0002_auto_20160406_1548'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='photo',
            options={'ordering': ('-post_time',)},
        ),
        migrations.AlterField(
            model_name='photo',
            name='fetch_time',
            field=models.DateTimeField(blank=True, help_text="The time the item's data was last fetched.", null=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='fetch_time',
            field=models.DateTimeField(blank=True, help_text='The time the data was last fetched.', null=True),
        ),
        migrations.AlterField(
            model_name='user',
            name='iconserver',
            field=models.PositiveIntegerField(default=0),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0002_auto_20150626_1521'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='bookmark',
            unique_together=set([('account', 'url')]),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import django.core.validators


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0003_auto_20150626_1558'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bookmark',
            name='url',
            field=models.TextField(validators=[django.core.validators.URLValidator()]),
        ),
    ]



import datetime
import json
import os
import pytz

from django.conf import settings
from django.core.files import File

from ..models import Media, Tweet, User
from ...core.utils.downloader import DownloadException, filedownloader

# Classes that take JSON data from the Twitter API and create or update
# objects.
#
# They don't call the API at all. But UserSaver fetches avatar images.

# CLASSES HERE:
#
# SaveUtilsMixin
# UserSaver
# TweetSaver


class SaveUtilsMixin(object):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def _api_time_to_datetime(self, api_time, time_format='%a %b %d %H:%M:%S +0000 %Y'):
        """Change a text datetime from the API to a datetime with timezone.
        api_time is a string like 'Wed Nov 15 16:55:59 +0000 2006'.
        """
        return datetime.datetime.strptime(api_time, time_format).replace(
                                                            tzinfo=pytz.utc)


class UserSaver(SaveUtilsMixin, object):
    "Provides a method for creating/updating a User using data from the API."

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save_user(self, user, fetch_time, download_avatar=True):
        """With Twitter user data from the API, it creates or updates the User
        and returns the User object.

        Keyword arguments:
        user -- A dict of the data about a user from the API's JSON.
        fetch_time -- A datetime.
        download_avatar -- Boolean. Should user's profile pic be downloaded?

        Returns the User object.
        """
        raw_json = json.dumps(user)

        defaults = {
            'fetch_time': fetch_time,
            'raw': raw_json,
            'screen_name': user['screen_name'],
            'name': user['name'],
            'is_private': user['protected'],
            'is_verified': user['verified'],
            'profile_image_url_https': user['profile_image_url_https'],
        }

        # When ingesting tweets there are lots of fields the 'user' element
        # doesn't have, compared to the API:

        if 'created_at' in user:
            defaults['created_at'] = \
                                self._api_time_to_datetime(user['created_at'])

        # If there's a URL it will be a t.co shortened one.
        # So we go through the entities to find its expanded version.
        if 'url' in user and user['url']:
            user_url = user['url']
            if 'url' in user['entities'] and 'urls' in user['entities']['url']:
                for url_dict in user['entities']['url']['urls']:
                    if url_dict['url'] == user['url'] and url_dict['expanded_url'] is not None:
                        user_url = url_dict['expanded_url']
            defaults['url'] = user_url

        if 'description' in user:
            defaults['description'] = user['description'] if user['description'] else ''

        if 'location' in user:
            defaults['location'] = user['location'] if user['location'] else ''

        if 'time_zone' in user:
            defaults['time_zone'] = user['time_zone'] if user['time_zone'] else ''

        if 'favourites_count' in user:
            defaults['favourites_count'] = user['favourites_count']

        for a_count in ['followers_count', 'friends_count', 'listed_count', 'statuses_count']:
            if a_count in user:
                defaults[a_count] = user[a_count]

        user_obj, created = User.objects.update_or_create(
            twitter_id=user['id'], defaults=defaults
        )

        if download_avatar:
            user_obj = self._fetch_and_save_avatar(user_obj)

        return user_obj

    def _fetch_and_save_avatar(self, user):
        """
        Download and save the Avatar/profile pic for this user.
        If the user's profile_image_url_https property doesn't match an image
        we've already downloaded for them, we fetch and save it.

        user -- User object.
        """
        filename = os.path.basename(user.profile_image_url_https)
        # Where we'd save the user's avatar image to:
        avatar_upload_path = os.path.join(
                                settings.MEDIA_ROOT,
                                user.avatar_upload_path(filename)
                            )
        if os.path.exists(avatar_upload_path):
            return user

        # We don't have this image yet, so fetch and save it.
        try:
            avatar_filepath = filedownloader.download(
                        user.profile_image_url_https,
                        ['image/jpeg', 'image/jpg', 'image/png', 'image/gif'])

            user.avatar.save(os.path.basename(avatar_filepath),
                                File(open(avatar_filepath, 'rb')) )
        except DownloadException as e:
            pass

        return user


class TweetSaver(SaveUtilsMixin, object):
    """Provides a method for creating/updating a Tweet (and its User) using
    data from the API. Also used by ingest.TweetIngester()
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def save_media(self, tweet):
        """Takes a Tweet object and creates or updates any photos and videos
        based on the JSON data in its `raw` field.

        Keyword arguments:
        tweet -- The Tweet object. Must have been saved as we need its id.

        Returns:
        Total number of items for this Tweet (regardless of whether they were
            created or updated).
        """
        import itertools

        # What we'll return:
        media_count = 0

        try:
            json_data = json.loads(tweet.raw)
        except ValueError as error:
            return media_count

        try:
            media = json_data['extended_entities']['media']
        except KeyError:
            return media_count

        for item in media:
            # Things common to photos, animated GIFs and videos.

            defaults = {
                'media_type':   'photo',
                'twitter_id':   item['id'],
                'image_url':    item['media_url_https'],
            }

            valid_types = [type for type,name in Media.MEDIA_TYPES]

            if item['type'] in valid_types:
                defaults['media_type'] = item['type']

            for size in ['large', 'medium', 'small', 'thumb']:
                if size in item['sizes']:
                    defaults[size+'_w'] = item['sizes'][size]['w']
                    defaults[size+'_h'] = item['sizes'][size]['h']
                else:
                    defaults[size+'_w'] = None
                    defaults[size+'_h'] = None

            # Adding things ony used for videos and animated GIFs:
            if 'video_info' in item:

                info = item['video_info']

                # eg, '16:9'
                defaults['aspect_ratio'] = '%s:%s' % (
                        info['aspect_ratio'][0], info['aspect_ratio'][1])

                if 'duration_millis' in info:
                    defaults['duration'] = info['duration_millis']

                # Group the variants into a dict with keys like 'video/mp4'
                # and values being a list of dicts. Each of those dicts
                # being a single variant.
                # [
                #   {'video/mp4': [{'url':'...', 'bitrate':320000},
                #                  {'url':'...', 'bitrate':832000},]
                #   },
                # ]
                # A bit less necessary now that mp4s are deprecated for videos.
                # (But there's still a single(?) mp4 url for animated GIFs.)

                # Sort the list of dicts by 'content_type'.
                # Need to do this for the next grouping stage.
                sorted_variants = sorted(
                            info['variants'], key=lambda k: k['content_type'])
                grouped_variants = {}

                for key, variants in itertools.groupby(sorted_variants, lambda k: k['content_type']):
                    grouped_variants[key] = list(variants)

                if 'application/dash+xml' in grouped_variants:
                    defaults['dash_url'] = grouped_variants['application/dash+xml'][0]['url']
                if 'application/x-mpegURL' in grouped_variants:
                    defaults['xmpeg_url'] = grouped_variants['application/x-mpegURL'][0]['url']

                if defaults['media_type'] == 'animated_gif' and 'video/mp4' in grouped_variants:
                    # Only Animated GIFs have mp4s now.
                    # https://twittercommunity.com/t/retiring-mp4-video-output/66093
                    defaults['mp4_url'] = grouped_variants['video/mp4'][0]['url']

            media_obj, created = Media.objects.update_or_create(
                    twitter_id=item['id'],
                    defaults=defaults
                )
            media_count += 1

            if tweet not in media_obj.tweets.all():
                media_obj.tweets.add(tweet)

        return media_count

    def save_tweet(self, tweet, fetch_time):
        """Takes a dict of tweet data from the API and creates or updates a
        Tweet object and its associated User object.

        Keyword arguments:
        tweet -- The tweet data.
        fetch_time -- A datetime.

        Returns:
        The Tweet object that was created or updated.
        """
        raw_json = json.dumps(tweet)
        try:
            created_at = self._api_time_to_datetime(tweet['created_at'])
        except ValueError:
            # Because the tweets imported from a downloaded archive have a
            # different format for created_at. Of course. Why not?!
            created_at = self._api_time_to_datetime(tweet['created_at'], time_format='%Y-%m-%d %H:%M:%S +0000')

        user = UserSaver().save_user(tweet['user'], fetch_time)

        if 'full_text' in tweet:
            # For new (2016) 'extended' format tweet data.
            # https://dev.twitter.com/overview/api/upcoming-changes-to-tweets
            text = tweet['full_text']
            frm = tweet['display_text_range'][0]
            to = tweet['display_text_range'][1]
            title = text[frm:to]
        else:
            # Older 'classic' format tweet data.
            text = tweet['text']
            title = text

        defaults = {
            'fetch_time':       fetch_time,
            'raw':              raw_json,
            'user':             user,
            'is_private':       user.is_private,
            'post_time':        created_at,
            'permalink':        'https://twitter.com/%s/status/%s' % (
                                                user.screen_name, tweet['id']),
            'title':            title.replace('\n', ' ').replace('\r', ' '),
            'text':             text,
            'twitter_id':       tweet['id'],
            'source':           tweet['source']
        }

        # Some of these are only present in tweets from the API (not in the
        # tweets from the downloaded archive).
        # Some are just not always present, such as coordinates and place stuff.

        if 'favorite_count' in tweet:
            defaults['favorite_count'] = tweet['favorite_count']

        if 'retweet_count' in tweet:
            defaults['retweet_count'] = tweet['retweet_count']

        if 'lang' in tweet:
            defaults['language'] = tweet['lang']

        if 'coordinates' in tweet and tweet['coordinates'] and 'type' in tweet['coordinates']:
            if tweet['coordinates']['type'] == 'Point':
                defaults['latitude'] = tweet['coordinates']['coordinates'][1]
                defaults['longitude'] = tweet['coordinates']['coordinates'][0]
            # TODO: Handle Polygons?

        if 'in_reply_to_screen_name' in tweet and tweet['in_reply_to_screen_name']:
            defaults['in_reply_to_screen_name'] =  tweet['in_reply_to_screen_name']
        else:
            defaults['in_reply_to_screen_name'] = ''

        if 'in_reply_to_status_id' in tweet:
            defaults['in_reply_to_status_id'] = tweet['in_reply_to_status_id']

        if 'in_reply_to_user_id' in tweet:
            defaults['in_reply_to_user_id'] = tweet['in_reply_to_user_id']

        if 'place' in tweet and tweet['place'] is not None:
            if 'attributes' in tweet['place'] and 'street_address' in tweet['place']['attributes']:
                defaults['place_attribute_street_address'] = tweet['place']['attributes']['street_address']
            if 'full_name' in tweet['place']:
                defaults['place_full_name'] = tweet['place']['full_name']
            if 'country' in tweet['place']:
                defaults['place_country'] = tweet['place']['country']

        if 'quoted_status_id' in tweet:
            defaults['quoted_status_id'] = tweet['quoted_status_id']

            if 'quoted_status' in tweet:
                # If tweet 1 quotes tweet 2 that quotes tweet 3, then
                # tweet 2 will have 'quoted_status_id' but not 'quoted_status'.
                # But the tweet does have quoted_status, we'll create/update
                # the quoted User object, and quoted Tweet.
                quoted_user = UserSaver().save_user(
                                    tweet['quoted_status']['user'], fetch_time)
                quoted_tweet_obj = self.save_tweet(
                                            tweet['quoted_status'], fetch_time)

        if 'retweeted_status' in tweet:
            defaults['retweeted_status_id'] = tweet['retweeted_status']['id']

            retweeted_user = UserSaver().save_user(
                                tweet['retweeted_status']['user'], fetch_time)
            retweeted_tweet_obj = self.save_tweet(
                                        tweet['retweeted_status'], fetch_time)

        tweet_obj, created = Tweet.objects.update_or_create(
                twitter_id=tweet['id'],
                defaults=defaults
            )

        # Create/update any Photos, and update the Tweet's photo_count:
        media_count = self.save_media(tweet=tweet_obj)
        tweet_obj.media_count = media_count

        tweet_obj.save()

        return tweet_obj




# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-03 14:57
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0011_auto_20160502_1645'),
    ]

    operations = [
        migrations.AlterField(
            model_name='photoset',
            name='flickr_id',
            field=models.BigIntegerField(db_index=True, unique=True),
        ),
        migrations.AlterField(
            model_name='photoset',
            name='photo_count',
            field=models.PositiveIntegerField(default=0, help_text='The number of photos in the set on Flickr'),
        ),
        migrations.AlterField(
            model_name='photoset',
            name='video_count',
            field=models.PositiveIntegerField(default=0, help_text='The number of videos in the set on Flickr'),
        ),
        migrations.AlterField(
            model_name='user',
            name='nsid',
            field=models.CharField(db_index=True, max_length=50, unique=True, verbose_name='NSID'),
        ),
    ]




from urllib.parse import quote_plus


def slugify_name(name):
    """
    Takes a string and returns the version used in a Last.fm URL
    As best as we can work out how its done.

    We need to do this for Albums, because their URLs aren't provided in the
    RecentScrobbles API response.

    See tests.lastfm.test_utils for examples.
    """

    # Last.fm doesn't quote certain punctuation characters:
    name = quote_plus(name, safe='!&(),:;')

    # Some things need extra encoding for Last.fm's URLs:
    replacements = (
        ('%2B', '%252B'),   # + has an extra level of quoting.
        ('%5C', '%5C%5C'),  # \ gets encoded twice.
    )

    for find, repl in replacements:
        name = name.replace(find, repl)

    return name




from django.conf.urls import url

from . import views


app_name = 'pinboard'

urlpatterns = [
    url(
        regex=r"^$",
        view=views.HomeView.as_view(),
        name='home'
    ),
    url(
        regex=r"^to-read/$",
        view=views.ToreadListView.as_view(),
        name='toread'
    ),
    url(
        regex=r"^tags/$",
        view=views.TagListView.as_view(),
        name='tag_list'
    ),
    url(
        regex=r"^tags/(?P<slug>[^/]+)/$",
        view=views.TagDetailView.as_view(),
        name='tag_detail'
    ),
    url(
        regex=r"^(?P<username>\w+)/$",
        view=views.AccountDetailView.as_view(),
        name='account_detail'
    ),
    url(
        regex=r"^(?P<username>\w+)/to-read/$",
        view=views.AccountToreadView.as_view(),
        name='account_toread'
    ),
    url(
        regex=r"^(?P<username>\w+)/tags/(?P<tag_slug>[^/]+)/$",
        view=views.AccountTagDetailView.as_view(),
        name='account_tag_detail'
    ),
    url(
        regex=r"^(?P<username>\w+)/(?P<hash>\w+)/$",
        view=views.BookmarkDetailView.as_view(),
        name='bookmark_detail'
    ),
]




# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-29 16:05
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0008_auto_20160429_1559'),
    ]

    operations = [
        migrations.AddField(
            model_name='photoset',
            name='fetch_time',
            field=models.DateTimeField(blank=True, help_text="The time the item's data was last fetched.", null=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0004_auto_20150626_1603'),
    ]

    operations = [
        migrations.AlterField(
            model_name='bookmark',
            name='summary',
            field=models.CharField(help_text=b"eg, Initial text of a blog post, start of the description of a photo, all of a Tweet's text, etc.", max_length=255, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-03 14:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0041_remove_media_tweet'),
    ]

    operations = [
        migrations.AlterField(
            model_name='media',
            name='tweets',
            field=models.ManyToManyField(related_name='media', to='twitter.Tweet'),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0008_remove_user_twitter_id_str'),
    ]

    operations = [
        migrations.AddField(
            model_name='account',
            name='last_fetch_id',
            field=models.BigIntegerField(help_text=b'The Twitter ID of the most recent Tweet fetched.', null=True, blank=True),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0016_auto_20151002_1525'),
    ]

    operations = [
        migrations.AddField(
            model_name='account',
            name='is_active',
            field=models.BooleanField(help_text="If false, new Tweets won't be fetched.", default=True),
        ),
    ]



import os

from django.core.files import File

from . import FetchError
from ..models import Photo
from ...core.utils.downloader import DownloadException, filedownloader


# A single class that fetches original photo/video files for existing
# Photo objects. Doesn't use the API.


class OriginalFilesFetcher(object):
    """
    Fetch the original photo files for a single Account.

    Not based off FlickrFetcher because we don't use the API so it's quite
    different. But still has a similar external appearance.

    Use something like:

        results = OriginalFilesFetcher(account=account_object).fetch()

    results is a dict that will have:
        'success': Boolean.
        'account': String. Indicating the Account (eg, its User's username).
        'fetched': Integer. If success, the number of files fetched, if any.
        'messages': List of strings. If no success, the failure message(s).
    """

    def __init__(self, account):
        self.account = None

        self.results = []

        self.results_count = 0

        self.return_value = {'fetched': 0}

        if account.user:
            self.return_value['account'] = account.user.username
        else:
            self.return_value['success'] = False
            self.return_value['messages'] = ['This account has no Flickr User']

        self.account = account

    def fetch(self, fetch_all=False):
        """
        Download and save original photos and videos for all Photo objects
        (or just those that don't already have them).

        self.account must be an Account object first.

        fetch_all -- Boolean. Fetch ALL photos/videos, even if we've already
                        got them?
        """
        # Might already have success=False from __init__():
        if 'success' not in self.return_value:
            self._fetch_files(fetch_all)

            self.return_value['fetched'] = self.results_count

        return self.return_value

    def _fetch_files(self, fetch_all):
        """
        Download and save original photos and videos for all Photo objects
        (or just those that don't already have them).

        fetch_all -- Boolean. Fetch ALL photos/videos, even if we've already
                        got them?
        """

        photos = Photo.objects.filter(user=self.account.user)

        if not fetch_all:
            photos = photos.filter(original_file='')

        error_messages = []

        for photo in photos:
            try:
                self._fetch_and_save_file(photo=photo, media_type='photo')
                self.results_count += 1
            except FetchError as e:
                error_messages.append(str(e))

            if photo.media == 'video':
                try:
                    self._fetch_and_save_file(photo=photo, media_type='video')
                    self.results_count += 1
                except FetchError as e:
                    error_messages.append(str(e))

        if len(error_messages) > 0:
            self.return_value['success'] = False
            self.return_value['messages'] = error_messages
        else:
            self.return_value['success'] = True

    def _fetch_and_save_file(self, photo, media_type):
        """
        Downloads a video or photo file and saves it to the Photo object.

        Expects:
            photo -- A Photo object.
            media_type -- String, either 'photo' or 'video'.

        Raises FetchError if something goes wrong.
        """

        if media_type == 'video':
            url = photo.remote_video_original_url
            # Accepted video formats:
            # https://help.yahoo.com/kb/flickr/sln15628.html
            # BUT, they all seem to be sent as video/mp4.
            acceptable_content_types = ['video/mp4',]

        else:
            url = photo.remote_original_url
            acceptable_content_types = [
                        'image/jpeg', 'image/jpg', 'image/png', 'image/gif',]

        filepath = False
        try:
            # Saves the file to /tmp/:
            filepath = filedownloader.download(url, acceptable_content_types)
        except DownloadException as e:
            raise FetchError(e)

        if filepath:
            # Reopen file and save to the Photo:
            reopened_file = open(filepath, 'rb')
            django_file = File(reopened_file)

            if media_type == 'video':
                photo.video_original_file.save(
                                    os.path.basename(filepath), django_file)
            else:
                photo.original_file.save(
                                    os.path.basename(filepath), django_file)





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Account',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('time_created', models.DateTimeField(help_text=b'The time this item was created in the database.', auto_now_add=True)),
                ('time_modified', models.DateTimeField(help_text=b'The time this item was last saved to the database.', auto_now=True)),
                ('screen_name', models.CharField(max_length=20, blank=True)),
                ('consumer_key', models.CharField(max_length=255, blank=True)),
                ('consumer_secret', models.CharField(max_length=255, blank=True)),
                ('access_token', models.CharField(max_length=255, blank=True)),
                ('access_token_secret', models.CharField(max_length=255, blank=True)),
            ],
            options={
                'ordering': ['screen_name'],
            },
        ),
    ]



# coding: utf-8
import argparse

from django.core.management.base import CommandError

from ....core.management.commands import DittoBaseCommand
from ...models import Account


class FetchTwitterCommand(DittoBaseCommand):

    # What we're fetching:
    singular_noun = 'Tweet'
    plural_noun = 'Tweets'

    # Child classes should supply some help text for the --recent argument:
    recent_help = ""

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            '--recent',
            action='store',
            default=False,
            help=self.recent_help
        )
        parser.add_argument(
            '--account',
            action='store',
            default=False,
            help='Only fetch for one Twitter account.',
        )

    def handle(self, *args, **options):

        # We might be fetching for a specific account or all (None).
        account = options['account'] if options['account'] else None;

        if options['recent']:
            # Will be either 'new' or a number; make the number an int.
            if options['recent'].isdigit():
                options['recent'] = int(options['recent'])

            results = self.fetch_tweets(account, options['recent'])
            self.output_results(results, options.get('verbosity', 1))
        elif options['account']:
            raise CommandError("Specify --recent as well as --account.")
        else:
            raise CommandError(
                        "Specify --recent, eg --recent=100 or --recent=new.")

    def fetch_tweets(self, screen_name, count):
        """Child classes should override this method to call a method that
        fetches tweets and returns results, eg:
            return RecentTweetsFetcher(screen_name=screen_name).fetch(count=count)
        """
        return {}


class UpdateTwitterCommand(DittoBaseCommand):
    """Parent class for commands which update Tweets/Users already in the DB.

    Child classes probably only need a fetch() method.
    """

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            '--account',
            action='store',
            default=False,
            help='Which Twitter account to use for the API call.',
        )

    def handle(self, *args, **options):
        if options['account']:
            screen_name = options['account']
            try:
                Account.objects.get(user__screen_name=screen_name)
            except Account.DoesNotExist:
                raise CommandError("There's no Account with a screen name of '%s'" % screen_name)
        else:
            raise CommandError("Specify --account, eg --account=philgyford.")

        results = self.fetch(screen_name)
        self.output_results(results, options.get('verbosity', 1))

    def fetch(self, screen_name):
        """Child classes should override this method to call a method that
        fetches Tweets/Users and returns results, eg:
            return TweetsFetcher(screen_name=screen_name).fetch()
        """
        return {}




from django.contrib import admin
from django.db import models
from django.forms import Textarea, TextInput

from ..core.admin import DittoItemModelAdmin
from .models import Account, Media, Tweet, User


@admin.register(Account)
class AccountAdmin(admin.ModelAdmin):
    list_display = ('user', 'is_active', 'time_created', 'time_modified',)

    fieldsets = (
        (None, {
            'fields': ('user', 'is_active', )
        }),
        ('API', {
            'fields': ('consumer_key', 'consumer_secret', 'access_token',
                        'access_token_secret',),
            'description': 'Keys and secrets require creation of an app at <a href="https://apps.twitter.com/">apps.twitter.com</a>',
        }),
        ('Data', {
            'fields': ('last_recent_id', 'last_favorite_id', 'time_created', 'time_modified',)
        }),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {'widget': TextInput(
            attrs={
                'class': 'vLargeTextField',
            }
        )},
    }

    readonly_fields = ('time_created', 'time_modified',)


class TweetsMediaInline(admin.TabularInline):
    model = Media.tweets.through
    extra = 0
    raw_id_fields = ('media', 'tweet',)


@admin.register(Media)
class MediaAdmin(admin.ModelAdmin):

    list_display = ('show_thumb', 'media_type', 'time_created', )
    list_display_links = ('show_thumb', )
    list_filter = ('time_created', )
    date_hierarchy = 'time_created'

    fieldsets = (
        (None, {
            'fields': ('show_image', 'media_type', 'twitter_id', 'image_url',
                        'image_file', 'mp4_file', ),
        }),
        ('Sizes', {
            'classes': ('collapse',),
            'fields': ('large_w', 'large_h', 'medium_w', 'medium_h', 'small_w', 'small_h', 'thumb_w', 'thumb_h', )
        }),
        ('Video', {
            'classes': ('collapse',),
            'fields': ('aspect_ratio', 'duration',
                        'mp4_url', 'dash_url', 'xmpeg_url',)
        }),
        ('Data', {
            'classes': ('collapse',),
            'fields': ('time_created', 'time_modified',)
        }),
    )

    inlines = [
        TweetsMediaInline,
    ]

    readonly_fields = ('show_image', 'time_created', 'time_modified',)
    exclude = ('tweets',)

    def show_thumb(self, instance):
        return '<img src="%(url)s" width="%(w)s" height="%(h)s" />' %\
                {
                    'url':  instance.thumbnail_url,
                    'w':    instance.thumbnail_w,
                    'h':    instance.thumbnail_h,
                }
    show_thumb.allow_tags = True
    show_thumb.short_description = ''

    def show_image(self, instance):
        if instance.media_type == 'photo':
            return '<img src="%(url)s" width="%(w)s" height="%(h)" />' %\
                {
                    'url':  instance.small_url,
                    'w':    instance.small_w,
                    'h':    instance.small_h,
                }
        else:
            return '<video width="%(w)s" height="%(h)s" poster="%(img)s" controls preload="metadata"><source src="%(video)s" type="%(mime)s"></video>' %\
                {
                    'w':        instance.small_w,
                    'h':        instance.small_h,
                    'img':      instance.small_url,
                    'video':    instance.video_url,
                    'mime':     instance.video_mime_type,
                }
    show_image.allow_tags = True
    show_image.short_description = 'Image'


@admin.register(Tweet)
class TweetAdmin(DittoItemModelAdmin):
    list_display = ('user','is_private', 'title', 'post_time', )
    list_display_links = ('title', )
    list_filter = ('post_time', 'fetch_time', )
    search_fields = ('text', )

    fieldsets = (
        (None, {
            'fields': ('user', 'text', 'text_html', 'title', 'summary',
                'post_time', 'post_year_str',
                'is_private', 'twitter_id', 'permalink', )
        }),
        (None, {
            'fields': ('favorite_count', 'retweet_count', 'media_count',
                'language', 'source',
                'in_reply_to_screen_name', 'in_reply_to_status_id',
                'in_reply_to_user_id',
                'quoted_status_id', 'retweeted_status_id',)
        }),
        ('Location', {
            'classes': ('collapse',),
            'fields': ('latitude', 'longitude',
                'place_attribute_street_address', 'place_full_name',
                'place_country',)
        }),
        ('Data', {
            'classes': ('collapse',),
            'fields': ('raw', 'fetch_time', 'time_created', 'time_modified',)
        }),
    )

    readonly_fields = ('post_year_str', 'text_html', 'raw',
                        'fetch_time', 'time_created', 'time_modified',)

    inlines = [
        TweetsMediaInline,
    ]

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {'widget': TextInput(
            attrs={
                'class': 'vLargeTextField',
            }
        )},
        # Reduce the number of rows.
        models.TextField: {'widget': Textarea(
            attrs={
                'class': 'vLargeTextField',
                'cols': 40,
                'rows': 5,
            }
        )},
    }


@admin.register(User)
class UserAdmin(admin.ModelAdmin):
    list_display = ('show_avatar', 'screen_name', 'name', 'is_private',
                    'fetch_time',)
    list_display_links = ('show_avatar', 'screen_name', )
    search_fields = ('screen_name', 'name',)

    fieldsets = (
        (None, {
            'fields': ('screen_name', 'name',
                'is_private', 'is_verified',
                'profile_image_url_https', 'avatar',
                'url', 'description', 'description_html', 'location',
                'time_zone', 'twitter_id', 'created_at', )
        }),
        ('Data', {
            'fields': ('raw', 'fetch_time', 'time_created', 'time_modified',)
        }),
    )

    formfield_overrides = {
        # Make the inputs full-width.
        models.CharField: {'widget': TextInput(
            attrs={
                'class': 'vLargeTextField',
            }
        )},
    }

    readonly_fields = ('description_html' ,'raw', 'fetch_time', 'time_created', 'time_modified',)

    def show_avatar(self, instance):
        return '<img src="%s" width="24" height="24" />' % (instance.avatar_url)
    show_avatar.allow_tags = True
    show_avatar.short_description = ''



from django.db import models

from ..core.managers import PublicItemManager


class PublicFavoritesManager(models.Manager):
    "Returns public Tweets favorited by any public Accounts."
    def get_queryset(self):
        from .models import User
        # All public Users associated with Accounts:
        users = User.objects.filter(account__isnull=False).filter(
                                                            is_private=False)
        return super().get_queryset().filter(is_private=False).filter(
                                        favoriting_users__in=users).distinct()


class FavoritesManager(models.Manager):
    "Returns public AND PRIVATE Tweets favorited by any of the Accounts."
    def get_queryset(self):
        from .models import User
        # All Users associated with Accounts:
        users = User.objects.filter(account__isnull=False)
        return super().get_queryset().filter(favoriting_users__in=users).distinct()


class TweetsManager(models.Manager):
    """Returns public AND PRIVATE Tweets posted by one of the Users with
    Accounts here.
    As opposed to just Tweets, which includes Tweets by any User that
    have been favorited by a User with an Account.
    """
    def get_queryset(self):
        from .models import User
        users = User.objects_with_accounts.all()
        return super().get_queryset().filter(user__in=users)


class PublicTweetsManager(PublicItemManager):
    """Returns public Tweets posted by one of the Users with Accounts here.
    As opposed to just public Tweets, which includes Tweets by any User that
    have been favorited by a User with an Account.
    """
    def get_queryset(self):
        from .models import User
        users = User.objects_with_accounts.all()
        return super().get_queryset().filter(user__in=users)


class WithAccountsManager(models.Manager):
    "Returns Twitter Users who have an Account on Ditto."

    def get_queryset(self):
        from .models import Account
        accounts = Account.objects.exclude(user__isnull=True)
        user_ids = [account.user.id for account in accounts]
        return super().get_queryset().filter(pk__in=user_ids)




# coding: utf-8
import argparse

from django.core.management.base import CommandError

from ....core.management.commands import DittoBaseCommand
from ...fetch import AllBookmarksFetcher, DateBookmarksFetcher, RecentBookmarksFetcher, UrlBookmarksFetcher


class Command(DittoBaseCommand):
    """Fetches bookmarks from Pinboard

    Fetch all bookmarks, from all accounts:
    ./manage.py fetch_pinboard_bookmarks --all

    Fetch bookmarks posted on one date:
    ./manage.py fetch_pinboard_bookmarks --date=2015-06-20

    Fetch the 20 most recent bookmarks:
    ./manage.py fetch_pinboard_bookmarks --recent=20

    Fetch the bookmark for one URL:
    ./manage.py fetch_pinboard_bookmarks --url=http://new-aesthetic.tumblr.com/

    Restrict any of the above to one account by adding the account's username:
    ./manage.py fetch_pinboardbookmarks --all --account=philgyford
    """

    singular_noun = 'Bookmark'
    plural_noun = 'Bookmarks'

    help = "Fetches bookmarks from Pinboard"

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            '--all',
            action='store_true',
            default=False,
            help='Fetch all bookmarks.'
        )
        parser.add_argument(
            '--date',
            action='store',
            default=False,
            help='Fetch bookmarks posted on one day, e.g. "2015-06-20".'
        )
        parser.add_argument(
            '--recent',
            action='store',
            default=False,
            help='Fetch the most recent bookmarks, e.g. "10".'
        )
        parser.add_argument(
            '--url',
            action='store',
            default=False,
            help='Fetch the bookmark for one URL, e.g. "http://www.foo.com".',
        )
        parser.add_argument(
            '--account',
            action='store',
            default=False,
            help='Only fetch for one Pinboard account.',
        )

    def handle(self, *args, **options):

        # We might be fetching for a specific account or all (None).
        account = options['account'] if options['account'] else None;

        if options['all']:
            results = AllBookmarksFetcher().fetch(username=account)

        elif options['date']:
            results = DateBookmarksFetcher().fetch(post_date=options['date'],
                                                            username=account)

        elif options['recent']:
            results = RecentBookmarksFetcher().fetch(num=options['recent'],
                                                            username=account)

        elif options['url']:
            results = UrlBookmarksFetcher().fetch(url=options['url'],
                                                            username=account)

        elif options['account']:
            raise CommandError("Specify --all, --recent, --date= or --url= as well as --account.")
        else:
            raise CommandError("Specify --all, --recent, --date= or --url=")

        self.output_results(results, options.get('verbosity', 1))




from django.conf.urls import url

from . import views


app_name = 'flickr'

urlpatterns = [
    url(
        regex=r"^$",
        view=views.HomeView.as_view(),
        name='home'
    ),
    url(
        regex=r"^albums/$",
        view=views.PhotosetListView.as_view(),
        name='photoset_list'
    ),
    url(
        regex=r"^tags/$",
        view=views.TagListView.as_view(),
        name='tag_list'
    ),
    url(
        regex=r"^tags/(?P<slug>[^/]+)/$",
        view=views.TagDetailView.as_view(),
        name='tag_detail'
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/$",
        view=views.UserDetailView.as_view(),
        name='user_detail'
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/albums/$",
        view=views.UserPhotosetListView.as_view(),
        name='user_photoset_list'
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/albums/(?P<flickr_id>[^/]+)/$",
        view=views.PhotosetDetailView.as_view(),
        name='photoset_detail'
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/tags/(?P<tag_slug>[^/]+)/$",
        view=views.UserTagDetailView.as_view(),
        name='user_tag_detail'
    ),
    url(
        regex=r"^(?P<nsid>[\dN@]+)/(?P<flickr_id>\d+)/$",
        view=views.PhotoDetailView.as_view(),
        name='photo_detail'
    ),
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0004_auto_20150730_1116'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tweet',
            name='favorite_count',
            field=models.PositiveIntegerField(default=0, help_text=b'Approximately how many times this has been favorited'),
        ),
        migrations.AlterField(
            model_name='tweet',
            name='retweet_count',
            field=models.PositiveIntegerField(default=0, help_text=b'Number of times this has been retweeted'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-06-03 13:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0038_tweet_retweeted_status_id'),
    ]

    operations = [
        migrations.AddField(
            model_name='media',
            name='tweets',
            field=models.ManyToManyField(to='twitter.Tweet'),
        ),
        migrations.AlterField(
            model_name='media',
            name='tweet',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='temporary_name', to='twitter.Tweet'),
        ),
    ]



from django.http import Http404
from django.shortcuts import get_object_or_404
from django.views.generic import DetailView
from django.views.generic.detail import SingleObjectMixin

from ..core.views import PaginatedListView
from .models import Account, Tweet, User


class HomeView(PaginatedListView):
    template_name = 'twitter/home.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account_list'] = Account.objects.all()
        return context

    def get_queryset(self):
        "Get Tweets by all of the Accounts that have Users."
        # Use select_related to fetch user details too. Could be nasty...
        return Tweet.public_tweet_objects.all().prefetch_related('user')


class FavoriteListView(PaginatedListView):
    template_name = 'twitter/favorite_list.html'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['account_list'] = Account.objects.all()
        return context

    def get_queryset(self):
        "Get Tweets by all of the Accounts that have Users."
        return Tweet.public_favorite_objects.all().prefetch_related('user')


class SingleUserMixin(SingleObjectMixin):
    """Used for views that need data about a User based on screen_name in
    the URL, and its Account if it has one.
    """
    slug_field = 'screen_name'
    slug_url_kwarg = 'screen_name'

    def get(self, request, *args, **kwargs):
        self.object = self.get_object(queryset=User.objects.all())
        return super().get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['twitter_user'] = self.object
        try:
            context['account'] = Account.objects.get(user=self.object)
        except Account.DoesNotExist:
            context['account'] = None
            context['public_accounts'] = Account.objects.filter(user__is_private=False)
        return context


class UserDetailView(SingleUserMixin, PaginatedListView):
    """A single Twitter User and its Tweets.
    The user might have an Account associated with it, or might not.
    """
    template_name = 'twitter/user_detail.html'

    def get_queryset(self):
        "All public tweets from this Account."
        return Tweet.public_objects.filter(user=self.object)\
                                                    .prefetch_related('user')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['tweet_list'] = context['object_list']
        return context


class AccountFavoriteListView(SingleUserMixin, PaginatedListView):
    "A single Twitter User associated with an Account, and its Favorites."
    template_name = 'twitter/account_favorite_list.html'

    def get_queryset(self):
        "All public favorites from this Account."
        return Tweet.public_favorite_objects.filter(
                        favoriting_users__in=[self.object])\
                                                    .prefetch_related('user')

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['tweet_list'] = context['object_list']
        return context


class TweetDetailView(DetailView):
    """Show a single tweet. It might be posted by one of the Accounts, or might
    be a tweet by someone else, favorited.
    """
    model = Tweet
    slug_field = 'twitter_id'
    slug_url_kwarg = 'twitter_id'

    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['twitter_user'] = context['tweet'].user
        if context['twitter_user'].is_private:
            # If private, we don't even send the Tweet to the template.
            context['tweet'] = None
        # We can show favorited tweets; they won't have an associated Account.
        try:
            context['account'] = Account.objects.get(
                                                user=context['twitter_user'])
        except Account.DoesNotExist:
            context['account'] = None
        return context




from django.core.management.base import BaseCommand, CommandError

from . import FetchCommand
from ...fetch.multifetchers import OriginalFilesMultiAccountFetcher


class Command(FetchCommand):
    """Fetches original photo files from Flickr.

    For all accounts:
        ./manage.py fetch_flickr_originals
        ./manage.py fetch_flickr_originals --all

    For one account:
        ./manage.py fetch_flickr_originals --account=35034346050@N01
        ./manage.py fetch_flickr_originals --account=35034346050@N01 --all
    """

    help = "Fetches the original image files for one or all Flickr Accounts"

    singular_noun = 'File'
    plural_noun = 'Files'

    def add_arguments(self, parser):
        super().add_arguments(parser)

        parser.add_argument(
            '--all',
            action='store_true',
            default=False,
            help="Fetch ALL files, even if they've been downloaded before. Otherwise, only fetch files that haven't already been downloaded."
        )

    def handle(self, *args, **options):
        # We might be fetching for a specific account or all (None).
        nsid = options['account'] if options['account'] else None;

        results = self.fetch_files(nsid, options['all'])
        self.output_results(results, options.get('verbosity', 1))

    def fetch_files(self, nsid, fetch_all=False):
        return OriginalFilesMultiAccountFetcher(nsid=nsid).fetch(
                                                        fetch_all=fetch_all)




import datetime
import pytz

from django import template
from django.db.models import Count

from ..models import Tweet, User
from ...core.utils import get_annual_item_counts


register = template.Library()

@register.simple_tag
def recent_tweets(screen_name=None, limit=10):
    """Returns a QuerySet of recent public Tweets, in reverse-chronological
    order.

    Keyword arguments:
    screen_name -- A Twitter user's screen_name. If not supplied, we fetch
                    Tweets for all Twitter users that have Accounts.
    limit -- Maximum number to fetch. Default is 10.
    """
    tweets = Tweet.public_tweet_objects.all()
    if screen_name is not None:
        tweets = tweets.filter(user__screen_name=screen_name)
    return tweets.prefetch_related('user')[:limit]

@register.simple_tag
def recent_favorites(screen_name=None, limit=10):
    """Returns a QuerySet of recent Tweets favorited by the Account associated
    with the Twitter User with screen_name.

    Keyword arguments:
    screen_name -- A Twitter user's screen_name. If not supplied, we fetch
                    Tweets favorited by all public Accounts.
    limit -- Maximum number to fetch. Default is 10.
    """
    if screen_name is None:
        tweets = Tweet.public_favorite_objects.all()
    else:
        user = User.objects.get(screen_name=screen_name)
        if user.is_private:
            tweets = Tweet.objects.none()
        else:
            tweets = Tweet.public_favorite_objects.filter(favoriting_users=user)
    return tweets.prefetch_related('user')[:limit]

@register.simple_tag
def day_tweets(date, screen_name=None):
    """Returns a QuerySet of Tweets posted on a specific date by public
    Accounts.

    Arguments:
    date -- A date object.

    Keyword arguments:
    screen_name -- A Twitter user's screen_name. If not supplied, we fetch
                    all public Tweets.
    """
    start = datetime.datetime.combine(date, datetime.time.min).replace(
                                                            tzinfo=pytz.utc)
    end   = datetime.datetime.combine(date, datetime.time.max).replace(
                                                            tzinfo=pytz.utc)
    tweets = Tweet.public_tweet_objects.filter(post_time__range=[start, end])
    if screen_name is not None:
        tweets = tweets.filter(user__screen_name=screen_name)
    tweets = tweets.prefetch_related('user')
    return tweets

@register.simple_tag
def day_favorites(date, screen_name=None):
    """Returns a QuerySet of Tweets posted on a specific date that have been
    favorited by public Accounts.

    NOTE: It is not the date on which the Tweets were favorited.
          The Twitter API doesn't supply that.

    Arguments:
    date -- A date object.

    Keyword arguments:
    screen_name -- A Twitter user's screen_name. If not supplied, we fetch
                    all public Tweets.
    """
    start = datetime.datetime.combine(date, datetime.time.min).replace(
                                                            tzinfo=pytz.utc)
    end   = datetime.datetime.combine(date, datetime.time.max).replace(
                                                            tzinfo=pytz.utc)
    if screen_name is None:
        tweets = Tweet.public_favorite_objects.filter(
                                                post_time__range=[start, end])
    else:
        user = User.objects.get(screen_name=screen_name)
        if user.is_private:
            tweets = Tweet.objects.none()
        else:
            tweets = Tweet.public_favorite_objects.filter(
                post_time__range=[start, end]).filter(favoriting_users=user)
    tweets = tweets.prefetch_related('user')
    return tweets


@register.simple_tag
def annual_tweet_counts(screen_name=None):
    """
    Get the number of public Tweets per year.
    Returns a list of dicts, sorted by year, like:
        [ {'year': 2015, 'count': 1234}, {'year': 2016, 'count': 9876} ]

    Keyword arguments:
    screen_name -- A Twitter user's screen_name. If not supplied, we fetch
                    all public Tweets.
    """

    tweets = Tweet.public_tweet_objects

    if screen_name is not None:
        tweets = tweets.filter(user__screen_name=screen_name)

    return get_annual_item_counts(tweets)


@register.simple_tag
def annual_favorite_counts(screen_name=None):
    """
    Get the number of public Favorites per year.
    (i.e. the Tweets are from those years, not that they were favorited then.)
    Returns a list of dicts, sorted by year, like:
        [ {'year': 2015, 'count': 1234}, {'year': 2016, 'count': 9876} ]

    Keyword arguments:
    screen_name -- A Twitter user's screen_name. If not supplied, we fetch
                    all public favorited Tweets.
    """

    if screen_name is None:
        tweets = Tweet.public_favorite_objects
    else:
        user = User.objects.get(screen_name=screen_name)
        if user.is_private:
            tweets = Tweet.objects.none()
        else:
            tweets = Tweet.public_favorite_objects.filter(favoriting_users=user)

    return get_annual_item_counts(tweets)




# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-10-26 15:40
from __future__ import unicode_literals
import random

from django.db import migrations, models


def lowercase_slugs(apps, schema_editor):
    """
    Populate the `original_slug` fields with the value of `slug`.
    Then convert all `slug`s to lowercase.

    A bit horrid. In case there are going to be duplicate lowercase slugs,
    we add a random number to the end of the next version.
    """

    Album = apps.get_model('lastfm', 'Album')
    slugs = []
    for row in Album.objects.all():
        row.original_slug = row.slug
        row.slug = row.slug.lower()
        if row.slug in slugs:
            row.slug = '%s+%s' % (row.slug, random.randrange(1, 1000000))
        slugs.append(row.slug)
        row.save()

    Artist = apps.get_model('lastfm', 'Artist')
    slugs = []
    for row in Artist.objects.all():
        row.original_slug = row.slug
        row.slug = row.slug.lower()
        if row.slug in slugs:
            row.slug = '%s+%s' % (row.slug, random.randrange(1, 1000000))
        slugs.append(row.slug)
        row.save()

    Track = apps.get_model('lastfm', 'Track')
    slugs = []
    for row in Track.objects.all():
        row.original_slug = row.slug
        row.slug = row.slug.lower()
        if row.slug in slugs:
            row.slug = '%s+%s' % (row.slug, random.randrange(1, 1000000))
        slugs.append(row.slug)
        row.save()


class Migration(migrations.Migration):

    dependencies = [
        ('lastfm', '0003_auto_20161026_1539'),
    ]

    operations = [
        migrations.RunPython(
            lowercase_slugs, reverse_code=migrations.RunPython.noop
        ),
        migrations.AlterField(
            model_name='album',
            name='original_slug',
            field=models.TextField(help_text='As used on Last.fm. Mixed case.'),
        ),
        migrations.AlterField(
            model_name='artist',
            name='original_slug',
            field=models.TextField(help_text='As used on Last.fm. Mixed case.', unique=True),
        ),
        migrations.AlterField(
            model_name='track',
            name='original_slug',
            field=models.TextField(help_text='As used on Last.fm. Mixed case.'),
        ),
    ]



from . import FetchError
from .fetchers import RecentPhotosFetcher, PhotosetsFetcher
from .filesfetchers import OriginalFilesFetcher
from ..models import Account, User

# Classes for fetching data from the API for ONE OR MORE Accounts.
# These are wrappers for other classes which do the heavy lifting.
# These call those Fetcher classes for each Account, and pass the results
# back.

# CLASSES HERE:
#
# MultiAccountFetcher
#   RecentPhotosMultiAccountFetcher
#   PhotosetsMultiAccountFetcher
#   OriginalFilesMultiAccountFetcher


class MultiAccountFetcher(object):
    """Parent class for fetching things from Flickr for ALL or ONE Account(s).

    Its child classes are useful for:
        * Fetching from all accounts.
        * Fetching from a single account with identical interface and response
          as when fetching from all accounts (otherwise you could use the
          single account fetcher, like RecentPhotosFetcher()).
        * Fetching from a single account when you only have the user's NSID.

    Use something like:

        results = ChildMultiAccountFetcher().fetch(foo=3)

    or:

        results = ChildMultiAccountFetcher(nsid='35034346050@N01').fetch(foo=3)
    """

    # Will be a list of Account objects.
    accounts = []

    def __init__(self, nsid=None):
        """Gets all of the Accounts that child classes will loop through.

        nsid -- If nsid is set, we use only the Account associated with the
                User with that nsid (if it's active). Otherwise, we use all
                active Accounts.
        """
        self.return_value = []

        if nsid is None:
            # Get all active Accounts.
            self.accounts = list(Account.objects.filter(is_active=True))
            if len(self.accounts) == 0:
                raise FetchError("No active Accounts were found to fetch.")
        else:
            # Find the Account associated with nsid.
            try:
                user = User.objects.get(nsid=nsid)
            except User.DoesNotExist:
                raise FetchError("There is no User with the NSID '%s'" % nsid)
            try:
                account = Account.objects.get(user=user)
            except Account.DoesNotExist:
                raise FetchError("There is no Account associated with the User with NSID '%s'" % nsid)
            if account.is_active == False:
                raise FetchError("The Account associated with the User with NSID '%s' is marked as inactive.")

            self.accounts = [account]
        return super().__init__()

    def fetch(self, **kwargs):
        raise FetchError(
            "Subclasess of MultiAccountFetcher should define their own fetch().")


class RecentPhotosMultiAccountFetcher(MultiAccountFetcher):
    """For fetching recent photos for ALL or ONE account(s).

    Usage:

        results = RecentPhotosMultiAccountFetcher().fetch(days=7)

    Or omit the `days` to fetch ALL photos for ALL accounts.

    results will be a list of dicts containing info about what was fetched (or
    went wrong) for each account.
    """

    def fetch(self, days=None):
        for account in self.accounts:
            self.return_value.append(
                RecentPhotosFetcher(account).fetch(days=days)
            )

        return self.return_value


class PhotosetsMultiAccountFetcher(MultiAccountFetcher):
    """For fetching ALL photosets for ALL or ONE account(s).

    Usage:

        results = PhotosetsMultiAccountFetcher().fetch()

    results will be a list of dicts containing info about what was fetched (or
    went wrong) for each account.
    """

    def fetch(self):
        for account in self.accounts:
            self.return_value.append(
                PhotosetsFetcher(account).fetch()
            )

        return self.return_value


class OriginalFilesMultiAccountFetcher(MultiAccountFetcher):
    """For fetching original photo files for ALL or ONE account(s).

    Usage:

        results = OriginalFilesMultiAccountFetcher().fetch(fetch_all=True)

    results will be a list of dicts containing info about what was fetched(or
    went wrong) for each account.
    """

    def fetch(self, fetch_all=False):
        for account in self.accounts:
            self.return_value.append(
                OriginalFilesFetcher(account).fetch(fetch_all=fetch_all)
            )

        return self.return_value




# -*- coding: utf-8 -*-
# Generated by Django 1.10.1 on 2016-11-17 16:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('flickr', '0019_auto_20160713_1127'),
    ]

    operations = [
        migrations.AlterField(
            model_name='photo',
            name='post_time',
            field=models.DateTimeField(blank=True, db_index=True, help_text='The time the item was originally posted/created on its service.', null=True),
        ),
        migrations.AlterField(
            model_name='photo',
            name='summary',
            field=models.CharField(blank=True, help_text="eg, Brief summary or excerpt of item's text content. No linebreaks or HTML.", max_length=255),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-04-28 15:25
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pinboard', '0018_auto_20160414_1637'),
    ]

    operations = [
        migrations.AddField(
            model_name='bookmark',
            name='url_hash',
            field=models.CharField(db_index=True, default='', help_text="Slug in the Bookmark's URL here.", max_length=12),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='bookmark',
            name='url',
            field=models.TextField(unique=True, validators=[django.core.validators.URLValidator()]),
        ),
    ]



#!/usr/bin/env python
import flickrapi

# Put your API Key and Secret here:
api_key = 'XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX'
api_secret = 'XXXXXXXXXXXXXXXX'

flickr = flickrapi.FlickrAPI(api_key, api_secret)

# Only do this if we don't have a valid token already
if not flickr.token_valid(perms='read'):

    # Get a request token
    flickr.get_request_token(oauth_callback='oob')

    # Open a browser at the authentication URL. Do this however
    # you want, as long as the user visits that URL.
    authorize_url = flickr.auth_url(perms='read')

    print("Open this URL in your web browser and, once you've signed in, and have agreed to authorization, you should get a code to type in:")
    print(authorize_url)

    # Get the verifier code from the user. Do this however you
    # want, as long as the user gives the application the code.
    verifier = input('Verifier code: ')

    # Trade the request token for an access token
    flickr.get_access_token(verifier)

    print("Done!")

else:
    print("This account is already authorised.")




# -*- coding: utf-8 -*-
# Generated by Django 1.9.5 on 2016-05-05 11:56
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twitter', '0035_auto_20160505_1137'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='media',
            name='mp4_bitrate_1',
        ),
        migrations.RemoveField(
            model_name='media',
            name='mp4_bitrate_2',
        ),
        migrations.RemoveField(
            model_name='media',
            name='mp4_bitrate_3',
        ),
        migrations.RemoveField(
            model_name='media',
            name='mp4_url_1',
        ),
        migrations.RemoveField(
            model_name='media',
            name='mp4_url_2',
        ),
        migrations.RemoveField(
            model_name='media',
            name='mp4_url_3',
        ),
        migrations.RemoveField(
            model_name='media',
            name='webm_bitrate',
        ),
        migrations.RemoveField(
            model_name='media',
            name='webm_url',
        ),
        migrations.AddField(
            model_name='media',
            name='mp4_url',
            field=models.URLField(blank=True, help_text='For Animated GIFs', verbose_name='MP4 URL'),
        ),
    ]



import os
import re
import requests
import shutil


class DownloadException(Exception):
    pass


class FileDownloader(object):
    """
    For downloading a file from a URL and saving it into /tmp/.

    Use like:
        from ditto.core.utils.downloader import filedownloader
        filepath = filedownloader.download(my_url, ['image/jpg'])

    filepath would be like '/tmp/image.jpg'
    """

    path = '/tmp/'

    def download(self, url, acceptable_content_types):
        """
        Downloads a file from a URL and saves it into /tmp/.
        Returns the filepath.

        Expects:
            url -- The URL of the file to fetch.
            acceptable_content_types -- A list of MIME types the request must
                match. eg:['image/jpeg', 'image/jpg', 'image/png', 'image/gif']

        Raises DownloadException if something goes wrong.
        """
        try:
            # From http://stackoverflow.com/a/13137873/250962
            r = requests.get(url, stream=True)
            if r.status_code == 200:
                try:
                    if r.headers['Content-Type'] in acceptable_content_types:
                        # Where we'll temporarily save the file:
                        filename = self.make_filename(url, r.headers)
                        filepath = '%s%s' % (self.path, filename)
                        # Save the file there:
                        with open(filepath, 'wb') as f:
                            r.raw.decode_content = True
                            shutil.copyfileobj(r.raw, f)
                        return filepath

                    else:
                        raise DownloadException(
                                "Invalid content type (%s) when fetching %s"%\
                                            (r.headers['content_type'], url))
                except KeyError:
                    raise DownloadException(
                        "No content_type headers found when fetching %s" % url)
            else:
                raise DownloadException(
                                "Got status code %s when fetching %s" % \
                                                        (r.status_code, url))
        except requests.exceptions.RequestException as e:
            raise DownloadException(
                        "Something when wrong when fetching %s: %s" % (url, e))

    def make_filename(self, url, headers={}):
        """
        Find the filename of a downloaded file.
        Returns a string.

        url -- The URL of the file that's been downloaded.
        headers -- A dict of response headers from requesting the URL.

        url will probably end in something like 'filename.jpg'.
        If not, we'll try and use the filename from the Content-Disposition
        header.
        This is the case for Videos we download from Flickr.
        """
        # Should work for photos:
        filename = os.path.basename(url)

        if filename == '':
            # Probably a Flickr video, so we have to get the filename from
            # headers:
            try:
                # Could be like 'attachment; filename=26897200312.avi'
                disposition = headers['Content-Disposition']
                m = re.search(
                            'filename\=(.*?)$', headers['Content-Disposition'])
                try:
                    filename = m.group(1)
                except (AttributeError, IndexError):
                    pass
            except KeyError:
                pass

        return filename

filedownloader = FileDownloader()

